#! /usr/bin/env python


"""This module provides routines to impute menage level variable to erfs_fpr data.
These variables are loyer and is some case zone_apl.

There are two ways of doing that:
 - using an external IPP imputation with iNSEE enquête logement done using stata
 - performing an inputation using R StatMatch library using rpy2 (to be extensively tested)

"""


# This step requires to have R installed with StatMatch library
# You'll also need rpy2 2.3x available at
# https://bitbucket.org/lgautier/rpy2/src/511312d70346f3f66c989e35443b2823e9b56d56?at=version_2.3.x
# (the version on python website is not compatible, working correctly for the debian's testing version)


import gc
import logging
import numpy
import os
import pandas as pd

# problem with rpy last version is not working
try:
    import rpy2
    import rpy2.robjects.pandas2ri
except ImportError:
    rpy2 = None

from openfisca_france_data.utils import (
    assert_variable_in_range, count_NA)
from openfisca_survey_manager.temporary import temporary_store_decorator
from openfisca_survey_manager import matching
from openfisca_survey_manager.statshelpers import mark_weighted_percentiles
from openfisca_survey_manager.survey_collections import SurveyCollection


log = logging.getLogger(__name__)


@temporary_store_decorator(file_name = 'erfs_fpr')
def merge_imputation_loyer(stata_file = None, temporary_store = None, year = None):
    assert year is not None
    assert stata_file is not None
    menages = temporary_store['menages_{}'.format(year)]
    logement = pd.read_stata(stata_file, preserve_dtypes = False, encoding = 'latin1')
    logement['ident12'] = logement.ident12.astype('int')
    logement.rename(columns = dict(zone_apl_EL = 'zone_apl', loyer_EL = 'loyer', ident12 = 'ident'), inplace = True)
    logement = logement[['zone_apl', 'loyer', 'ident']]
    menages = menages.merge(
        logement,
        left_on = 'ident',
        right_on = 'ident',
        how = 'left',
        )
    menages.rename(columns = dict(so = 'statut_occupation_logement'), inplace = True)
    temporary_store['menages_{}'.format(year)] = menages
    return menages


@temporary_store_decorator(file_name = 'erfs_fpr')
def imputation_loyer(temporary_store = None, year = None):
    assert temporary_store is not None
    assert year is not None

    kind = 'erfs_fpr'
    erf = create_comparable_erf_data_frame(temporary_store = temporary_store, year = year)
    logement = create_comparable_logement_data_frame(temporary_store = temporary_store, year = year)

    logement = logement.loc[logement.lmlm.notnull()].copy()
    log.info("Dropping {} observations form logement".format(logement.lmlm.isnull().sum()))

    if kind == 'erfs_fpr':
        allvars = [
            'deci',
            'hnph2',
            'magtr',
            'mcs8',
            'mdiplo',
            'mtybd',
            'statut_occupation',
            ]
    else:
        allvars = [
            'deci',
            'hnph2',
            'iaat_bis',
            'magtr',
            'mcs8',
            'mdiplo',
            'mtybd',
            'statut_occupation',
            'tu99_recoded',
            ]

    # TODO keep the variable indices

    erf = erf[allvars + ['ident', 'wprm']].copy()

    for variable in allvars:
        erf_unique_values = set(erf[variable].unique())
        logement_unique_values = set(logement[variable].unique())
        if not erf_unique_values <= logement_unique_values:
            print('''
                {} span wrong
                erf: {},
                logement: {}
                concerns {} observations
                '''.format(
                    variable,
                    erf_unique_values,
                    logement_unique_values,
                    erf[variable].isin(erf_unique_values - logement_unique_values).sum()
                    )
                )

    if kind == 'erfs_fpr':
        log.info("dropping {} erf observations".format(len(erf.query('mtybd == 0 | mcs8 == 0'))))
        erf = erf.query('mtybd != 0 & mcs8 != 0').copy()

    else:
        log.info("dropping {} erf observations".format(len(erf.query('iaat_bis == 0 | mtybd == 0 | mcs8 == 0'))))
        erf = erf.query('iaat_bis != 0 & mtybd != 0 & mcs8 != 0').copy()

    for variable in allvars:
        erf_unique_values = set(erf[variable].unique())
        logement_unique_values = set(logement[variable].unique())
        assert erf_unique_values <= logement_unique_values

    if kind == 'erfs_fpr':
        classes = "deci"
    else:
        classes = ['deci', 'tu99_recoded']

    matchvars = list(set(allvars) - set(classes))

    fill_erf_nnd, fill_erf_nnd_1 = matching.nnd_hotdeck_using_rpy2(
        receiver = erf,
        donor = logement,
        matching_variables = matchvars,
        z_variables = "lmlm",
        donor_classes = classes,
        )

    fill_erf_nnd.rename(columns = {'lmlm': 'loyer'}, inplace = True)

    loyers_imputes = fill_erf_nnd[['ident', 'loyer']].copy()
    menages = temporary_store['menages_{}'.format(year)]
    for loyer_var in ['loyer_x', 'loyer_y', 'loyer']:
        if loyer_var in menages.columns:
            del menages[loyer_var]

    menages = menages.merge(loyers_imputes, on = 'ident', how = 'left')
    assert 'loyer' in menages.columns, "La variable loyer n'est pas présente dans menages"

    temporary_store['menages_{}'.format(year)] = menages
    return


# Helpers


def prepare_erf_menage(temporary_store = None, year = None, kind = None):
    """
    Préparation de la sous-table ERFS de travail
    """
    assert temporary_store is not None
    assert year is not None

    log.info("Préparation de la table ménage de l'ERF")
    if kind == "erfs_fpr":
        # revenus_variables = [
        #     "chomage",
        #     "rag",
        #     "retraites",
        #     "rev_etranger"
        #     "rev_financier_prelev_lib_imputes",
        #     "rev_fonciers_bruts",
        #     "rev_valeurs_mobilieres_bruts",
        #     "ric",
        #     "rnc",
        #     ]
        revenus_variables = ["revdecm"]
        localisation_variables = [
            "pol10",
            "tau10",
            "tu10",
            ]
        nationalite_variables = [
            "nfr"
            ]
    else:
        revenus_variables = [
            "zperm",
            "zracm",
            "zragm",
            "zricm",
            "zrncm",
            "ztsam",
            ]
        localisation_variables = [
            "aai1",  # aai1: Année achèvement de l'immeuble (8 postes); Année achèvement de l'immeuble aai2
            "pol99",
            "tau99",
            "tu99",
            ]
        nationalite_variables = [
            "nat28pr",
            ]

    menage_variables = revenus_variables + localisation_variables + [
        "agpr",
        "cstotpr",
        "ident",
        "nb_uci",
        "nbenfc",
        "nbpiec",
        "reg",
        "so",
        "spr",
        "typmen5",
        "wprm",
        ]

    if year == 2008:  # Tau99 not present
        menage_variables = menage_variables.pop('tau99')

    erf_menages = temporary_store['menages_{}'.format(year)]
    erf_menages = erf_menages[menage_variables].copy()

    if kind == "erfs_fpr":
        erf_menages['revtot'] = 0  # TODO change this
    else:
        erf_menages['revtot'] = (
            erf_menages.zperm +
            erf_menages.zragm +
            erf_menages.zricm +
            erf_menages.zrncm +
            erf_menages.ztsam +
            erf_menages.zracm
            )

    # Niveau de vie de la personne de référence
    erf_menages['nvpr'] = erf_menages.revtot.astype('float') / erf_menages.nb_uci.astype('float')
    erf_menages.loc[erf_menages.nvpr < 0, 'nvpr'] = 0
    erf_menages.rename(columns = dict(so = 'statut_occupation'), inplace = True)

    log.info("Preparation de la tables individus de l'ERF")
    erfindm = temporary_store['individus_{}'.format(year)]
    erfindm = erfindm.loc[erfindm.lpr == 1, ['ident', 'dip11']].copy()

    log.info("Fusion des tables menage et individus de l'ERF")
    erf = erf_menages.merge(erfindm, on = 'ident', how = 'inner')
    del erf_menages, erfindm
    assert not erf.duplicated().any(), \
        "Il y a {} ménages dupliqués".format(erf.duplicated().sum())
    # erf = erf.drop_duplicates('ident')
    erf.rename(
        columns = {
            'dip11': 'mdiplo',
            'nat28pr': 'mnatio',
            'nbpiec': 'hnph2',
            },
        inplace = True,
        )
    # Extraction des seuls locataires
    erf = erf.loc[erf.statut_occupation.isin(range(3, 6))]
    gc.collect()

    return erf


def compute_decile(erf):
    dec, values = mark_weighted_percentiles(
        erf.nvpr.values,
        numpy.arange(1, 11),
        erf.wprm.values,
        2,
        return_quantiles = True
        )
    values.sort()
    erf['deci'] = (
        1 +
        (erf.nvpr > values[1]) +
        (erf.nvpr > values[2]) +
        (erf.nvpr > values[3]) +
        (erf.nvpr > values[4]) +
        (erf.nvpr > values[5]) +
        (erf.nvpr > values[6]) +
        (erf.nvpr > values[7]) +
        (erf.nvpr > values[8]) +
        (erf.nvpr > values[9])
        )
    assert erf.deci.isin(range(1, 11)).all()
    del dec, values
    gc.collect()


def format_variable_magtr(erf):
    """
    Format varaible magtr
    TODO describe using enquête logement
    """
    if (erf.agpr == '.').any():
        erf = erf[erf.agpr != '.'].copy()
    erf['agpr'] = erf.agpr.astype('int')
    erf['magtr'] = 3
    erf.loc[erf.agpr < 65, 'magtr'] = 2
    erf.loc[erf.agpr < 40, 'magtr'] = 1
    assert erf.magtr.isin(range(1, 5)).all()
    del erf['agpr']


def format_variable_mcs8(erf):
    assert erf.cstotpr.notnull().all()
    erf['mcs8'] = erf.cstotpr.astype('float') / 10.0
    erf.mcs8 = numpy.floor(erf.mcs8).astype('int')
    # TODO: assert erf.mcs8.isin(range(1, 9))
    del erf['cstotpr']
    assert erf.mcs8.isin(range(0, 9)).all()
    erf.loc[erf.mcs8.isin([4, 8]), 'mcs8'] = 4
    erf.loc[erf.mcs8.isin([5, 6, 7]), 'mcs8'] = 5
    # TODO: assert erf.mcs8.isin(range(1, 6)).all(), erf.mcs8.value_counts()
    assert erf.mcs8.isin(range(0, 6)).all()


def format_variable_mtybd(erf):
    erf['mtybd'] = 0
    erf.loc[(erf.typmen5 == 1) & (erf.spr != 2), 'mtybd'] = 1
    erf.loc[(erf.typmen5 == 1) & (erf.spr == 2), 'mtybd'] = 2
    erf.loc[erf.typmen5 == 5, 'mtybd'] = 3
    erf.loc[erf.typmen5 == 3, 'mtybd'] = 7
    erf.loc[erf.nbenfc == 1, 'mtybd'] = 4
    erf.loc[erf.nbenfc == 2, 'mtybd'] = 5
    erf.loc[erf.nbenfc >= 3, 'mtybd'] = 6
    erf.drop(
        ['nbenfc', 'spr', 'typmen5'],
        axis = 1,
        inplace = True,
        )
    assert erf.mtybd.isin(range(0, 8)).all()
    # TODO: assert erf.mtybd.isin(range(1,8)).all(),


def format_variable_hnph2(erf):
    # 3 logements ont 0 pièces !!
    erf.loc[erf.hnph2 < 1, 'hnph2'] = 1
    erf.loc[erf.hnph2 >= 6, 'hnph2'] = 6
    assert erf.hnph2.isin(range(1, 7)).all()
    # TODO: il reste un NA 2003
    #       il rest un NA en 2008  (Not rechecked)


def format_variable_mnatio(erf, kind = None):
    if kind == 'erfs_fpr':
        pass
    else:
        erf.loc[erf.mnatio == 10, 'mnatio'] = 1
        mnatio_range = range(11, 16) + range(21, 30) + range(31, 33) + range(41, 49) + range(51, 53) + [60] + [62]
        erf.loc[erf.mnatio.isin(mnatio_range), 'mnatio'] = 2
        assert erf.mnatio.isin(range(1, 3)).all(), 'valeurs pour mnatio\n{}'.format(erf.mnatio.value_counts())


def format_variable_iaat_bis(erf, kind = None):
    # iaat_bis is a contraction of iaat varaible of enquête logement
    # iaat_bis DATE D'ACHEVEMENT DE LA CONSTRUCTION
    # Voir http://www.insee.fr/fr/ppp/bases-de-donnees/fichiers_detail/eec10/doc/pdf/eec10_qlogement.pdf
    if kind == 'erfs_fpr':
        pass
    else:
        erf['iaat_bis'] = 0
        erf.loc[erf.aai1.isin([1, 2, 3]), 'iaat_bis'] = 1
        erf.loc[erf.aai1 == 4, 'iaat_bis'] = 2
        erf.loc[erf.aai1 == 5, 'iaat_bis'] = 3
        erf.loc[erf.aai1 == 6, 'iaat_bis'] = 4
        erf.loc[erf.aai1 == 7, 'iaat_bis'] = 5
        erf.loc[erf.aai1 == 8, 'iaat_bis'] = 6
        # TODO: assert erf.iaat_bis.isin(range(1, 7)).all()
        assert erf.iaat_bis.isin(range(0, 7)).all()


def format_variable_mdiplo(erf):
    erf.loc[erf.mdiplo.isin([71, ""]), 'mdiplo'] = 1
    erf.loc[erf.mdiplo.isin([70, 60, 50]), 'mdiplo'] = 2
    erf.loc[erf.mdiplo.isin([41, 42, 31, 33]), 'mdiplo'] = 3
    erf.loc[erf.mdiplo.isin([10, 11, 30]), 'mdiplo'] = 4
    # TODO: assert erf.mdiplo.isin(range(1, 5)).all()
    assert erf.mdiplo.isin(range(0, 5)).all()


def format_variable_tu99_recoded(erf, kind = None):
    if kind == 'erfs_fpr':
        # TODO use tu10
        pass
    else:
        erf['tu99_recoded'] = erf.tu99.copy()
        erf.loc[erf.tu99 == 0, 'tu99_recoded'] = 1
        erf.loc[erf.tu99.isin([1, 2, 3]), 'tu99_recoded'] = 2
        erf.loc[erf.tu99.isin([4, 5, 6]), 'tu99_recoded'] = 3
        erf.loc[erf.tu99 == 7, 'tu99_recoded'] = 4
        erf.loc[erf.tu99 == 8, 'tu99_recoded'] = 5
        assert erf.tu99_recoded.isin(range(1, 6)).all()
        del erf['tu99']


def check(erf, kind = None):
    erf.wprm = erf.wprm.astype('int')
    if kind == 'erfs_fpr':
        check_variables = [
            'hnph2',
            'magtr',
            'mcs8',
            'mdiplo',
            'mtybd',
            'statut_occupation',
            ]
    else:
        check_variables = [
            'hnph2',
            'iaat_bis',
            'magtr',
            'mcs8',
            'mdiplo',
            'mnatio',
            'mtybd',
            'statut_occupation',
            'tu99_recoded',
            ]
    gc.collect()

    non_integer_variables_to_filter = list()
    integer_variables_to_filter = list()
    for check_variable in check_variables:
        if erf[check_variable].isnull().any():
            log.info("Des valeurs NaN sont encore présentes dans la variable {} la table ERF".format(check_variable))
            non_integer_variables_to_filter.append(check_variable)

        if (erf[check_variable] == 0).any():
            if numpy.issubdtype(erf[check_variable].dtype, numpy.integer):
                log.info(
                    "Des valeurs nulles sont encore présentes dans la variable {} la table ERF".format(check_variable))
                integer_variables_to_filter.append(check_variable)

    assert len(non_integer_variables_to_filter) == 0, non_integer_variables_to_filter
    # assert len(integer_variables_to_filter) == 0, integer_variables_to_filter

    # On vérifie au final que l'on n'a pas de doublons d'individus
    assert not erf.ident.duplicated().any(), 'Il y a {} doublons'.format(erf.ident.duplicated().sum())
    return erf


def create_comparable_erf_data_frame(temporary_store = None, year = None):
    """
    Préparation des variables de la table ERFS qui serviront à l'imputation
    """
    assert temporary_store is not None
    assert year is not None

    erf = prepare_erf_menage(temporary_store = temporary_store, year = year, kind = "erfs_fpr")
    compute_decile(erf)  # deci = decile de revenu par unité de consommation

    # TODO: faire le lien avec men_vars,
    # il manque "pol99","reg","tau99" et ici on a en plus logt, 'nvpr','revtot','dip11','deci'
    variables = [
        'aai1',
        'agpr',
        'cstotpr',
        'deci',
        'dip11',
        'ident',
        'nat28pr',
        'nb_uci',
        'nbenfc',
        'nbpiec',
        'nvpr',
        'revtot',
        'spr',
        'statut_occupation',
        'tu99',
        'typmen5',
        'wprm',
        'zperm',
        'zracm',
        'zragm',
        'zricm',
        'zrncm',
        'ztsam',
        ]

    format_variable_magtr(erf)
    format_variable_mcs8(erf)
    format_variable_mtybd(erf)
    format_variable_hnph2(erf)

    format_variable_mnatio(erf, kind = 'erfs_fpr')
    format_variable_iaat_bis(erf, kind = 'erfs_fpr')
    format_variable_mdiplo(erf)
    format_variable_tu99_recoded(erf, kind = 'erfs_fpr')
    check(erf, kind = 'erfs_fpr')
    return erf


def create_comparable_logement_data_frame(temporary_store = None, year = None):
    assert temporary_store is not None
    assert year is not None
    logement_adresse_variables = ["gzc2"]
    logement_menage_variables = [
        "maa1at",
        "magtr",
        "mcs8",
        "mdiplo",
        "mrcho",
        "mrret",
        "mrsal",
        "mrtns",
        "mtybd",
        "muc1",
        "qex",
        "sec1",
        ]

    if year == 2003:
        logement_menage_variables.extend(["hnph2", "ident", "lmlm", "mnatior", "typse"])
        logement_adresse_variables.extend(["iaat", "ident", "tu99"])
    if year > 2005:  # and year < 2010:
        logement_menage_variables.extend(["idlog", "mnatio"])
        logement_adresse_variables.extend(["idlog"])  # pas de typse en 2006
        logement_logement_variables = ["hnph2", "iaat", "idlog", "lmlm", "tu99"]  # pas de typse en 2006

    # Travail sur la table logement
    # Table menage
    if year == 2003:
        year_lgt = 2003
    if year > 2005:  # and year < 2010:
        year_lgt = 2006

    logement_survey_collection = SurveyCollection.load(collection = 'logement')
    logement_survey = logement_survey_collection.get_survey('logement_{}'.format(year_lgt))

    log.info("Preparing logement menage table")

    try:
        logement_menage = logement_survey.get_values(
            table = "menage", variables = logement_menage_variables)
    except Exception:
        logement_menage = logement_survey.get_values(
            table = "menage1", variables = logement_menage_variables)

    logement_menage.rename(columns = {'idlog': 'ident'}, inplace = True)

    for revenus in ['mrcho', 'mrret', 'mrsal', 'mrtns']:
        logement_menage[revenus].fillna(0, inplace = True)

    logement_menage['revtot'] = (
        logement_menage.mrcho +
        logement_menage.mrret +
        logement_menage.mrsal +
        logement_menage.mrtns
        )
    # TODO : Virer les revenus négatifs ? mrtns :  118 revenus négatifs sur 42845 en 2006
    assert logement_menage.revtot.notnull().all()
    logement_menage['nvpr'] = 10.0 * logement_menage['revtot'] / logement_menage['muc1']

    assert logement_menage.qex.notnull().all()
    assert (logement_menage.qex > 0).all()

    dec, values = mark_weighted_percentiles(
        logement_menage['nvpr'].values,
        numpy.arange(1, 11),
        logement_menage['qex'].values,
        2,
        return_quantiles = True,
        )
    values.sort()
    logement_menage['deci'] = (
        1 +
        (logement_menage.nvpr > values[1]) +
        (logement_menage.nvpr > values[2]) +
        (logement_menage.nvpr > values[3]) +
        (logement_menage.nvpr > values[4]) +
        (logement_menage.nvpr > values[5]) +
        (logement_menage.nvpr > values[6]) +
        (logement_menage.nvpr > values[7]) +
        (logement_menage.nvpr > values[8]) +
        (logement_menage.nvpr > values[9])
        )

    del dec, values
    assert logement_menage['deci'].isin(range(1, 11)).all(), "Logement decile are out of range'"
    gc.collect()

    if year_lgt == 2006:
        log.info('Preparing logement logement table')
        try:
            lgtlgt = logement_survey.get_values(
                table = "lgt_logt", variables = logement_logement_variables)
        except Exception:
            lgtlgt = logement_survey.get_values(
                table = "logement", variables = logement_logement_variables)

        lgtlgt.rename(columns = {'idlog': 'ident'}, inplace = True)
        logement_menage = logement_menage.merge(lgtlgt, left_on = 'ident', right_on = 'ident', how = 'inner')
        del lgtlgt

    data = logement_menage.loc[logement_menage.sec1.isin([21, 22, 23, 24, 30])].copy()
    del logement_menage
    gc.collect()

    if year_lgt == 2006:
        data.rename(columns = {'mnatio': 'mnatior'}, inplace = True)

    data = data.loc[data.mnatior.notnull()].copy()
    data = data.loc[data.sec1.notnull()].copy()
    data['tmp'] = data.sec1.astype("int")
    data.loc[data.sec1.isin([21, 22, 23]), 'tmp'] = 3
    data.loc[data.sec1 == 24, 'tmp'] = 4
    data.loc[data.sec1 == 30, 'tmp'] = 5
    data['statut_occupation'] = data.tmp
    count_NA('statut_occupation', data)
    logement_menage = data[data.statut_occupation.notnull()].copy()

    # Table adresse
    log.info("Préparation de la table adresse de l'enquête logement")

    logement_adresse = logement_survey.get_values(table = "adresse", variables = logement_adresse_variables)
    logement_adresse.rename(columns = {'idlog': 'ident'}, inplace = True)

    log.info("Fusion des tables logement et ménage de l'enquête logement")
    Logement = logement_menage.merge(logement_adresse, on = 'ident', how = 'inner')

    Logement.loc[Logement.hnph2 >= 6, 'hnph2'] = 6
    Logement.loc[Logement.hnph2 < 1, 'hnph2'] = 1
    count_NA('hnph2', Logement)
    assert Logement.hnph2.notnull().any(), "Some hnph2 are null"
    # Logement=(Logement[Logement['hnph2'].notnull()]) # Mis en comment car 0 NA pour hnph2

    # On est dans la même étape within ici et par la suite ( cf code R )
    # TODO : ici problème je transforme les 07 en 7
    # car Python considère les 0n comme des nombres octaux ( < 08 ).
    # J'espère que ce n'est pas important.
    Logement.loc[Logement.mnatior.isin([0, 1]), 'mnatior'] = 1
    Logement.loc[Logement.mnatior.isin([2, 3, 4, 5, 6, 7, 8, 9, 10, 11]), 'mnatior'] = 2
    count_NA('mnatior', Logement)
    assert_variable_in_range('mnatior', [1, 3], Logement)

    Logement['iaat_bis'] = 0
    Logement.loc[Logement.iaat.isin([1, 2, 3, 4, 5]), 'iaat_bis'] = 1  # avant 1967
    Logement.loc[Logement.iaat == 6, 'iaat_bis'] = 2  # 1968 - 1974
    Logement.loc[Logement.iaat == 7, 'iaat_bis'] = 3  # 1975 - 1981
    Logement.loc[Logement.iaat == 8, 'iaat_bis'] = 4  # 1982 - 1989
    Logement.loc[Logement.iaat == 9, 'iaat_bis'] = 5  # 1990 - 1998
    Logement.loc[Logement.iaat == 10, 'iaat_bis'] = 6  # après 1999
    assert Logement.iaat_bis.isin(range(1, 7)).all()

    Logement.loc[Logement.mdiplo == 1, 'mdiplo'] = 1
    Logement.loc[Logement.mdiplo.isin([2, 3, 4]), 'mdiplo'] = 2
    Logement.loc[Logement.mdiplo.isin([5, 6, 7, 8]), 'mdiplo'] = 3
    Logement.loc[Logement.mdiplo == 9, 'mdiplo'] = 4
    Logement.loc[Logement.mdiplo.isnull(), 'mdiplo'] = 0
    # TODO: assert Logement.mdiplo.isin(range(1, 5)).all()
    assert Logement.mdiplo.isin(range(0, 5)).all()
    Logement.mdiplo = Logement.mdiplo.astype('int')

    Logement.loc[Logement.mtybd == 110, 'mtybd'] = 1
    Logement.loc[Logement.mtybd == 120, 'mtybd'] = 2
    Logement.loc[Logement.mtybd == 200, 'mtybd'] = 3
    Logement.loc[Logement.mtybd.isin([311, 321, 401]), 'mtybd'] = 4
    Logement.loc[Logement.mtybd.isin([312, 322, 402]), 'mtybd'] = 5
    Logement.loc[Logement.mtybd.isin([313, 323, 403]), 'mtybd'] = 6
    Logement.loc[Logement.mtybd == 400, 'mtybd'] = 7
    assert Logement.mtybd.isin(range(1, 8)).all()
    Logement.mtybd = Logement.mtybd.astype('int')

    Logement['tu99_recoded'] = Logement.tu99.copy()
    count_NA('tu99', Logement)
    Logement.loc[Logement.tu99 == 0, 'tu99_recoded'] = 1
    Logement.loc[Logement.tu99.isin([1, 2, 3]), 'tu99_recoded'] = 2
    Logement.loc[Logement.tu99.isin([4, 5, 6]), 'tu99_recoded'] = 3
    Logement.loc[Logement.tu99 == 7, 'tu99_recoded'] = 4
    Logement.loc[Logement.tu99 == 8, 'tu99_recoded'] = 5
    count_NA('tu99_recoded', Logement)
    assert_variable_in_range('tu99_recoded', [1, 6], Logement)

    Logement.loc[Logement.gzc2 == 1, 'gzc2'] = 1
    Logement.loc[Logement.gzc2.isin([2, 3, 4, 5, 6]), 'gzc2'] = 2
    Logement.loc[Logement.gzc2 == 7, 'gzc2'] = 3
    count_NA('gzc2', Logement)
    # TODO: assert_variable_in_range('gzc2', [1, 4], Logement)

    Logement.loc[Logement.magtr.isin([1, 2]), 'magtr'] = 1
    Logement.loc[Logement.magtr.isin([3, 4]), 'magtr'] = 2
    Logement.loc[Logement.magtr == 5, 'magtr'] = 3
    assert Logement.magtr.isin(range(1, 4)).all()

    # Logement.loc[Logement.mcs8 == 1, 'mcs8'] = 1
    # Logement.loc[Logement.mcs8 == 2, 'mcs8'] = 2
    # Logement.loc[Logement.mcs8 == 3, 'mcs8'] = 3
    Logement.loc[Logement.mcs8.isin([4, 8]), 'mcs8'] = 4
    Logement.loc[Logement.mcs8.isin([5, 6, 7]), 'mcs8'] = 5
    assert Logement.mcs8.isin(range(1, 6)).all()

    Logement['logloy'] = numpy.log(Logement['lmlm'].values)
    kept_variables = [
        'deci',
        'hnph2',
        'iaat_bis',
        'lmlm',
        'magtr',
        'mcs8',
        'mdiplo',
        'mtybd',
        'qex',
        'statut_occupation',
        'tu99_recoded',
        # 'ident',
        ]

    logement = Logement[kept_variables].copy()
    # logement.rename(columns = {'qex': 'wprm'}, inplace = True)
    return logement


if __name__ == '__main__':
    import sys
    logging.basicConfig(level = logging.INFO, stream = sys.stdout)
    year = 2012
    from openfisca_france_data.erfs_fpr.input_data_builder import step_01_preprocessing
    step_01_preprocessing.build_merged_dataframes(year = year)
    openfisca_survey_collection = SurveyCollection(name = 'openfisca')
    output_data_directory = openfisca_survey_collection.config.get('data', 'output_directory')
    stata_file = os.path.join(output_data_directory, 'log_men_ERFS.dta')
    menages = merge_imputation_loyer(stata_file = stata_file, year = year)



import gc
import logging
import numpy as np
import pandas as pd

from openfisca_survey_manager.temporary import temporary_store_decorator  # type: ignore
from openfisca_france_data.utils import assert_dtype
from openfisca_france_data.smic import smic_horaire_brut


log = logging.getLogger(__name__)

AGE_RSA = 25


@temporary_store_decorator(file_name = 'erfs_fpr')
def build_famille(temporary_store = None, year = None):
    """
    Création des familles
    Création des variables 'idfam' and 'quifam'
    """

    assert temporary_store is not None
    assert year is not None

    skip_enfants_a_naitre = True
    kind = 'erfs_fpr'

    log.info(' [1] Preparing the data base')
    log.info(' [1.1] Getting individuals')

    indivi = temporary_store['individus_{}'.format(year)]

    # Si on pense qu'on ne peut pas récupérer les enfants à naître
    if skip_enfants_a_naitre:
        log.info(" [1.2] Skipping enfants à naître")
        enfants_a_naitre = None
    else:
        if year < 2013:
            lpr = "lpr"
        else:
            lpr = "lprm"
        lien = 'lien' if year < 2013 else 'lienprm'  # TODO: attention pas les mêmes modalités
        log.info(" [1.2] Récupération des enfants à naître")
        cohab = 'cohab' if year < 2013 else "coured"
        agepr = 'agepr' if year < 2013 else "ageprm"
        retrai = 'retrai' if year < 2013 else "ret"
        individual_variables = [
            'acteu',
            'actrec',
            'agepf',
            agepr,
            cohab,
            'contra',
            'declar1',
            'forter',
            'ident',
            lien,
            lpr,
            'mrec',
            'naia',
            'naim',
            'noi',
            'noicon',
            'noidec',
            'noimer',
            'noindiv',
            'persfip',
            'quelfic',
            retrai,
            'rga',
            'rstg',
            'sexe',
            'stc',
            'titc',
            'year',
            'ztsai',
            'wprm',
            ] + (["noiper"] if "noiper" in indivi.columns else [])

        enfants_a_naitre = temporary_store['enfants_a_naitre_{}'.format(year)][individual_variables].copy()
        enfants_a_naitre.drop_duplicates('noindiv', inplace = True)

        ean_avant = len(enfants_a_naitre.index)
        enfants_a_naitre = enfants_a_naitre.loc[enfants_a_naitre[lpr] == 3].copy()
        enfants_a_naitre = enfants_a_naitre.loc[~(enfants_a_naitre.noindiv.isin(indivi.noindiv.values))].copy()
        ean_apres = len(enfants_a_naitre.index)

        log.debug("Il y a {} ({}) enfants à naitre avant (après) avoir retiré ceux qui ne sont pas enfants de la personne de référence".format(ean_avant, ean_apres))

    individus = create_familles(indivi = indivi, year = year, kind = kind, enfants_a_naitre = enfants_a_naitre,
        skip_enfants_a_naitre = skip_enfants_a_naitre)

    temporary_store['individus_{}'.format(year)] = individus


# steps

def create_familles(indivi = None, year = None, kind = 'erfs_fpr', enfants_a_naitre = None,
        skip_enfants_a_naitre = True):
    """
    On essaie de repérer un type de famille, à chaque fois.
    Le réliquat on le met où on peut.

    TODO : jeter un oeil à comment INES l'a fait
    """
    assert indivi is not None
    assert year is not None
    assert (enfants_a_naitre is not None) or skip_enfants_a_naitre

    complete_indivi(indivi, year)

    base = famille_1(
        indivi = indivi,
        kind = kind,
        enfants_a_naitre = enfants_a_naitre,
        skip_enfants_a_naitre = True,
        year = year,
        )

    base, famille, personne_de_reference = famille_2(
        base = base,
        year = year,
        )

    base, famille = famille_3(
        base = base,
        famille = famille,
        kind = kind,
        year = year,
        )

    base, famille = famille_5(
        base = base,
        famille = famille,
        kind = kind,
        year = year,
        )

    base, famille = famille_6(
        base = base,
        famille = famille,
        personne_de_reference = personne_de_reference,
        kind = kind,
        year = year,
        )

    # here 17000919 is removed bc. "no chef de famille" - but famille df is not returned
    # so these obs are still in the famille data. return famille df as well? check implications
    # or simply remove manually afterwarfs/workaround
    individus = famille_7(
        base = base,
        famille = famille,
        indivi = indivi,
        kind = kind,
        year = year,
        )

    return individus


def complete_indivi(indivi, year):
    indivi['year'] = year
    # indivi["noidec"] = indivi["declar1"].str[0:2].copy()  # Not converted to int because some NaN are present

    indivi["agepf"] = (
        (indivi.naim < 7) * (indivi.year - indivi.naia) +
        (indivi.naim >= 7) * (indivi.year - indivi.naia - 1)
        )

    # Enfant en nourrice exclus
    # before 2013, lien ==6 inclut enfants en nourrice sans lien de parenté
    # en 2014 lienprm == 6 inclut tous les "sans lien de parenté". On va dire que c'est équivalent

    try:
        selection_enfant_en_nourrice = (indivi.lien == 6) & (indivi.agepf < 16)
    except Exception:
        selection_enfant_en_nourrice = (indivi.lienprm == 6) & (indivi.agepf < 16)

    nb_enfants_en_nourrice = selection_enfant_en_nourrice.sum()

    if nb_enfants_en_nourrice > 0:
        indivi = indivi[~(selection_enfant_en_nourrice)].copy()
        log.debug("{} enfants en nourrice sont exlus".format(nb_enfants_en_nourrice.sum()))

    # for series_name in ['agepf']:  # , 'noidec']:  # integer with NaN
    #     assert_dtype(indivi[series_name], "object")


def famille_1(indivi = None, kind = 'erfs_fpr', enfants_a_naitre = None, skip_enfants_a_naitre = True, year = None):
    """
    Préparation de la base de travail
    """
    assert indivi is not None
    assert (enfants_a_naitre is not None) or (skip_enfants_a_naitre)
    assert year is not None
    # TODO check if we can remove acteu forter etc since dealt with in 01_pre_proc
    smic = smic_horaire_brut[year]

    # PB with vars "agepf"  "noidec" "year"  NOTE: quels problèmes ? JS

    log.info(" [1.3] création de la base complète")

    if skip_enfants_a_naitre:
        base = indivi.copy()
    else:
        base = pd.concat([indivi, enfants_a_naitre])

    log.debug("base contient {} lignes ".format(len(base.index)))

    base['noindiv'] = (100 * base.ident + base['noi']).astype(int)
    base['moins_de_15_ans_inclus'] = base.agepf < 16
    base['jeune_non_eligible_rsa'] = (base.agepf >= 16) & (base.agepf < AGE_RSA)
    base['jeune_eligible_rsa'] = base.agepf >= AGE_RSA

    if kind == 'erfs_fpr':
        base['salaire_de_base'].fillna(0, inplace = True)
        base['smic55'] = base.salaire_de_base >= (smic * 151 * 12 * 0.55)  # 55% du smic mensuel brut
    else:
        base['ztsai'].fillna(0, inplace = True)
        base['smic55'] = base.ztsai >= (smic * 151 * 12 * 0.55)  # 55% du smic mensuel brut

    base['famille'] = 0
    base['kid'] = False

    for series_name in ['kid', 'moins_de_15_ans_inclus', 'jeune_non_eligible_rsa', 'jeune_eligible_rsa', 'smic55']:
        assert_dtype(base[series_name], "bool")
    try:
        assert_dtype(base.famille, "int")
    except Exception:
        assert_dtype(base.famille, "int64")

    # TODO: remove or clean from NA assert_dtype(base.ztsai, "int")
    return base


def famille_2(base, year = None):
    assert year is not None
    log.info(" [2] On cherche les enfants ayant père et/ou mère comme personne de référence et conjoint")

    if year < 2013:
        lpr = "lpr"
    else:
        lpr = "lprm"

    personne_de_reference = base.loc[base[lpr] == 1, ['ident', 'noi']].copy()
    personne_de_reference['noifam'] = (100 * personne_de_reference.ident + personne_de_reference['noi']).astype(int)
    personne_de_reference = personne_de_reference[['ident', 'noifam']].copy()

    log.debug("length personne_de_reference : {}".format(len(personne_de_reference.index)))

    nof01 = base.loc[
        base[lpr].isin([1, 2]) |
        ((base[lpr] == 3) & (base.moins_de_15_ans_inclus)) |
        ((base[lpr] == 3) & base.jeune_non_eligible_rsa & (~base.smic55))
        ].copy()
    log.debug('longueur de nof01 avant merge: {}'.format(len(nof01.index)))
    nof01 = nof01.merge(personne_de_reference, on = 'ident', how = 'outer')
    nof01['famille'] = 10
    nof01['kid'] = (
        ((nof01[lpr] == 3) & (nof01.moins_de_15_ans_inclus)) |
        ((nof01[lpr] == 3) & (nof01.jeune_non_eligible_rsa) & ~(nof01.smic55))
        )
    for series_name in ['famille', 'noifam']:
        try:
            assert_dtype(nof01[series_name], "int")
        except Exception:
            assert_dtype(nof01[series_name], "int64")

    assert_dtype(nof01.kid, "bool")
    famille = nof01.copy()
    del nof01
    control_04(famille, base)

    log.info(" [2.1] Identification des couples non personne de référence ou conjoint de celle-ci")

    # On adopte une approche non genrée
    couple = subset_base(base, famille)
    cohab = 'cohab' if year < 2013 else "coured"
    couple = couple[(couple[cohab] == 1) & (couple[lpr] >= 3)].copy()
    # l'identifiant famille est lié au noi de le plus bas
    couple['noifam'] = np.minimum(
        (100 * couple.ident + couple.noi).astype(int),
        (100 * couple.ident + couple.noicon).astype(int)
        )

    # Recherche la présence de polyandes/polygames
    noindiv_conjoint = (100 * base.ident + base.noicon).astype(int)
    conjoints = base.loc[
        base.noindiv.isin(noindiv_conjoint),
        ['noindiv', 'ident', 'noi', lpr, 'noicon']
        ]
    if conjoints.noindiv.duplicated().sum() > 0:
        log.info("Nombre de personnes polyandres/polygames: {}".format(
            conjoints.duplicated().sum()
        ))
    del noindiv_conjoint

    couple['famille'] = 20
    for series_name in ['famille', 'noifam']:
        try:
            assert_dtype(couple[series_name], "int")
        except Exception:
            assert_dtype(couple[series_name], "int64")

    log.debug("Nombre de personnes vivant en couple sans être personne de référence ou conjoint: {} dont {} sans conjoints dans la base".format(
        len(couple), len(couple.loc[~((100 * couple.ident + couple.noicon).astype(int).isin(base.noindiv))])))

    famille = pd.concat([famille, couple])
    control_04(famille, base)
    return base, famille, personne_de_reference


def famille_3(base = None, famille = None, kind = 'erfs_fpr', year = None):

    if year < 2013:
        lpr = "lpr"
    else:
        lpr = "lprm"

    assert base is not None
    assert famille is not None
    assert year is not None

    log.info(" [3] Récupération des personnes seules")
    log.info(" [3.1] personnes seules de catégorie 1")

    seul1 = base[~(base.noindiv.isin(famille.noindiv.values))].copy()
    cohab = 'cohab' if year < 2013 else "coured"

    seul1 = seul1[
        (seul1[lpr].isin([3, 4,5,6])) & ((seul1.jeune_non_eligible_rsa & seul1.smic55) | seul1.jeune_eligible_rsa) & (seul1[cohab] == 1) &
        (seul1.sexe == 2)].copy()

    log.debug("Il y a {} personnes seules de catégorie 1".format(len(seul1.index)))

    if len(seul1.index) > 0:
        seul1['noifam'] = (100 * seul1.ident + seul1.noi).astype(int)
        seul1['famille'] = 31
        for series_name in ['famille', 'noifam']:
            assert_dtype(seul1[series_name], "int")
        famille = pd.concat([famille, seul1])
        control_04(famille, base)

    log.info(" [3.1] personnes seules de catégorie 2")
    seul2 = base[~(base.noindiv.isin(famille.noindiv.values))].copy()
    seul2 = seul2[(seul2[lpr].isin([3, 4,5,6])) & seul2.jeune_non_eligible_rsa & seul2.smic55 & (seul2[cohab] != 1)].copy()
    log.debug("Il y a {} personnes seules de catégorie 2".format(
        len(seul2.index)))
    if len(seul2.index) > 0:
        seul2['noifam'] = (100 * seul2.ident + seul2.noi).astype(int)
        seul2['famille'] = 32

        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(seul2[series_name], "int")
            except Exception:
                assert_dtype(seul2[series_name], "int64")
        famille = pd.concat([famille, seul2])
        control_04(famille, base)

    log.info(" [3.3] personnes seules de catégorie 3")
    seul3 = subset_base(base, famille)
    seul3 = seul3[(seul3[lpr].isin([3, 4,5,6])) & seul3.jeune_eligible_rsa & (seul3[cohab] != 1)].copy()
    # TODO: CHECK erreur dans le guide méthodologique ERF 2002 lpr 3,4 au lieu de 3 seulement
    log.debug("Il y a {} personnes seules de catégorie 3".format(
        len(seul3.index)))
    if len(seul3.index) > 0:
        seul3['noifam'] = (100 * seul3.ident + seul3.noi).astype(int)
        seul3['famille'] = 33
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(seul3[series_name], "int")
            except Exception:
                assert_dtype(seul3[series_name], "int64")
        famille = pd.concat([famille, seul3])
        control_04(famille, base)

    log.info(" [3.4] personnes seules de catégorie 4")
    seul4 = subset_base(base, famille)
    assert seul4.noimer.notnull().all()
    if kind == 'erfs_fpr':
        seul4 = seul4[
            (seul4[lpr].isin([4,5,6])) &
            seul4.jeune_non_eligible_rsa &
            (~(seul4.smic55)) &
            # (seul4.noiper == 0) &
            (seul4.noimer == 0)
            ].copy()
    else:
        seul4 = seul4[
            (seul4[lpr].isin([4,5,6])) &
            seul4.jeune_non_eligible_rsa &
            (~(seul4.smic55)) &
            # (seul4.noiper == 0) &
            (seul4.noimer == 0) &
            (seul4.persfip == 'vous')
            ].copy()
    log.debug("Il y a {} personnes seules de catégorie 4".format(
        len(seul4.index)))

    if len(seul4.index) > 0:
        seul4['noifam'] = (100 * seul4.ident + seul4.noi).astype(int)
        seul4['famille'] = 34
        famille = pd.concat([famille, seul4])
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(seul4[series_name], "int")
            except Exception:
                assert_dtype(seul4[series_name], "int64")

    control_04(famille, base)

    log.info(" [4] traitement des enfants")
    log.info(" [4.1] enfant avec mère")

    avec_mere = subset_base(base, famille)
    avec_mere = avec_mere[
        (avec_mere[lpr].isin([4,5,6])) &
        (avec_mere.jeune_non_eligible_rsa | avec_mere.moins_de_15_ans_inclus) &
        (avec_mere.noimer > 0)
        ].copy()
    log.debug("Il y a {} enfants rattachés à leur mère".format(len(avec_mere.index)))

    avec_mere['noifam'] = (100 * avec_mere.ident + avec_mere.noimer).astype(int)
    avec_mere['famille'] = 41
    avec_mere['kid'] = True
    if len(avec_mere.index) > 0:
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(avec_mere[series_name], "int")
            except Exception:
                assert_dtype(avec_mere[series_name], "int64")

    # On récupère les mères des enfants (elles peuvent avoir plusieurs enfants et il faut unicité de l'identifiant)
    mere = (
        pd.DataFrame(avec_mere['noifam'].copy())
        .rename(columns = {'noifam': 'noindiv'})
        .drop_duplicates()
        .merge(base)
        )
    log.debug("qui sont au nombre de {}".format(len(mere)))

    mere['noifam'] = (100 * mere.ident + mere.noi).astype(int)
    mere['famille'] = 42
    for series_name in ['famille', 'noifam']:
        try:
            assert_dtype(mere[series_name], "int")
        except Exception:
            assert_dtype(mere[series_name], "int64")

    avec_mere = avec_mere[avec_mere.noifam.isin(mere.noindiv)].copy()
    famille = famille[~(famille.noindiv.isin(mere.noindiv.values))].copy()  # Avoid duplication in famille

    # On retrouve les conjoints des mères
    assert mere.noicon.notnull().all()
    conjoint_mere_id = mere.loc[mere.noicon > 0, ['ident', 'noicon', 'noifam']].copy()
    conjoint_mere_id['noindiv'] = (100 * conjoint_mere_id.ident + conjoint_mere_id.noicon).astype(int)
    log.debug("et dont les conjoints sont au nombre de {}".format(len(conjoint_mere_id)))
    conjoint_mere = (conjoint_mere_id[['noindiv', 'noifam']].copy()
        .merge(base)
        )
    conjoint_mere['famille'] = 43
    for series_name in ['famille', 'noifam']:
        try:
            assert_dtype(conjoint_mere[series_name], "int")
        except Exception:
            assert_dtype(conjoint_mere[series_name], "int64")

    famille = famille[~(famille.noindiv.isin(conjoint_mere.noindiv.values))].copy()  # Avoid duplication in famille
    famille = pd.concat([famille, avec_mere, mere, conjoint_mere], sort = True)
    del avec_mere, mere, conjoint_mere, conjoint_mere_id
    control_04(famille, base)

    log.info(" [4.2] enfants avec père")
    avec_pere = subset_base(base, famille)
    if "noiper" in base.columns:
        assert avec_pere.noiper.notnull().all()
        avec_pere = avec_pere[
            (avec_pere[lpr].isin([4,5,6])) &
            (avec_pere.jeune_non_eligible_rsa | avec_pere.moins_de_15_ans_inclus) &
            (avec_pere.noiper > 0)
            ].copy()
        log.debug("Il y a {} enfants rattachés à leur père".format(
            len(avec_pere.index)))
        avec_pere['noifam'] = (100 * avec_pere.ident + avec_pere.noiper).astype(int)
        #Check if father is in the database
        avec_pere = avec_pere[(avec_pere.noifam.isin(base.noindiv))].copy()
        avec_pere['famille'] = 44
        avec_pere['kid'] = True
        assert avec_pere['noifam'].notnull().all(), 'presence of NaN in avec_pere'
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(avec_pere[series_name], "int")
            except Exception:
                assert_dtype(avec_pere[series_name], "int64")

        assert_dtype(avec_pere.kid, "bool")

        pere = (
            pd.DataFrame(avec_pere['noifam'])
            .rename(columns = {'noifam': 'noindiv'})
            .drop_duplicates()
            .merge(base)
            )
        pere['noifam'] = (100 * pere.ident + pere.noi).astype(int)
        pere['famille'] = 45
        famille = famille[~(famille.noindiv.isin(pere.noindiv.values))].copy()  # Avoid duplication in famille

        # On récupère les conjoints des pères
        assert pere.noicon.notnull().all()
        conjoint_pere_id = pere.loc[pere.noicon > 0, ['ident', 'noicon', 'noifam']].copy()
        log.debug("et dont les conjoints sont au nombre de {}".format(
            len(conjoint_pere_id)))

        if len(conjoint_pere_id.index) > 0:
            conjoint_pere_id['noindiv'] = (100 * conjoint_pere_id.ident + conjoint_pere_id.noicon).astype(int)
            conjoint_pere = (conjoint_pere_id[['noindiv', 'noifam']].copy()
                .merge(base)
                )
            conjoint_pere['famille'] = 46
            for series_name in ['famille', 'noifam']:
                try:
                    assert_dtype(conjoint_pere[series_name], "int")
                except Exception:
                    assert_dtype(conjoint_pere[series_name], "int64")

            famille = famille[~(famille.noindiv.isin(conjoint_pere.noindiv.values))].copy()  # Avoid duplication in famille
            famille = pd.concat([famille, avec_pere, pere, conjoint_pere])
        else:
            conjoint_pere = None
            famille = pd.concat([famille, avec_pere, pere], sort = True)
        control_04(famille, base)

        del avec_pere, pere, conjoint_pere, conjoint_pere_id
    if kind == 'erfs_fpr':
        log.info(" [4.3] enfants avec déclarant (ignorée dans erfs_fpr)")
        pass
    else:
        log.info(" [4.3] enfants avec déclarant")
        avec_dec = subset_base(base, famille)
        avec_dec = avec_dec[
            (avec_dec.persfip == "pac") &
            (avec_dec[lpr].isin([4,5,6])) &
            (
                (avec_dec.jeune_non_eligible_rsa & ~(avec_dec.smic55)) | (avec_dec.moins_de_15_ans_inclus == 1)
                )
            ]
        avec_dec['noifam'] = (100 * avec_dec.ident + avec_dec.noidec.astype('int')).astype('int')
        avec_dec['famille'] = 47
        avec_dec['kid'] = True
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(avec_dec[series_name], "int")
            except Exception:
                assert_dtype(avec_dec[series_name], "int64")
        assert_dtype(avec_dec.kid, "bool")
        control_04(avec_dec, base)
        # on récupère les déclarants pour leur attribuer une famille propre
        declarant_id = pd.DataFrame(avec_dec['noifam'].copy()).rename(columns={'noifam': 'noindiv'})
        declarant_id.drop_duplicates(inplace = True)
        dec = declarant_id.merge(base)
        dec['noifam'] = (100 * dec.ident + dec.noi).astype(int)
        dec['famille'] = 48
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(dec[series_name], "int")
            except Exception:
                assert_dtype(dec[series_name], "int64")
        famille = famille[~(famille.noindiv.isin(dec.noindiv.values))].copy()
        famille = pd.concat([famille, avec_dec, dec])
        del dec, declarant_id, avec_dec
        control_04(famille, base)

    return base, famille


def famille_5(base = None, famille = None, kind = 'erfs_fpr', year = None):
    assert base is not None
    assert famille is not None
    assert year is not None
    smic = smic_horaire_brut[year]
    lien = 'lien' if year < 2013 else 'lienprm'  # TODO: attention pas les mêmes modalités
    lpr = 'lpr' if year < 2013 else 'lprm'
    cohab = 'cohab' if year < 2013 else "coured"
    agepr = 'agepr' if year < 2013 else "ageprm"
    retrai = 'retrai' if year < 2013 else "ret"
    if kind == 'erfs_fpr':
        log.info(" [5] Récupération des enfants fip (ignorée dans erfs_fpr)")
        pass
    else:
        log.info(" [5] Récupération des enfants fip")
        log.info(" [5.1] Création de la df fip")
        individual_variables_fip = [
            'acteu',
            'actrec',
            'agepf',
            agepr,
            cohab,
            'contra',
            'declar1',
            'forter',
            'ident',
            lien,
            lpr,
            'mrec',
            'naia',
            'naim',
            'noi',
            'noicon',
            'noidec',
            'noimer',
            'noindiv',
            'persfip',
            'quelfic',
            retrai,
            'rga',
            'rstg',
            'sexe',
            'stc',
            'titc',
            'year',
            'ztsai',
            ] + (["noiper"] if "noiper" in base.columns else [])
        # TODO: temporary_store is not defined here
        fip = temporary_store['fipDat_{}'.format(year)][individual_variables_fip].copy()
        # Variables auxilaires présentes dans base qu'il faut rajouter aux fip'
        # WARNING les noindiv des fip sont construits sur les ident des déclarants
        # pas d'orvelap possible avec les autres noindiv car on a des noi =99, 98, 97 ,...'
        fip['moins_de_15_ans_inclus'] = (fip.agepf < 16)
        fip['jeune_non_eligible_rsa'] = ((fip.agepf >= 16) & (fip.agepf < AGE_RSA))
        fip['jeune_eligible_rsa'] = (fip.agepf >= AGE_RSA)
        fip['smic55'] = (fip.ztsai >= smic * 151 * 12 * 0.55)
        fip['famille'] = 0
        fip['kid'] = False
        for series_name in ['kid', 'moins_de_15_ans_inclus', 'jeune_non_eligible_rsa', 'jeune_eligible_rsa', 'smic55']:
            assert_dtype(fip[series_name], "bool")
        for series_name in ['famille']:
            try:
                assert_dtype(fip[series_name], "int")
            except Exception:
                assert_dtype(fip[series_name], "int64")

        log.info(" [5.2] extension de base avec les fip")
        base_ = pd.concat([base, fip])
        enfant_fip = subset_base(base_, famille)
        enfant_fip = enfant_fip[
            (enfant_fip.quelfic == "FIP") & (
                (enfant_fip.agepf.isin([19, 20]) & ~(enfant_fip.smic55)) |
                ((enfant_fip.naia == enfant_fip.year - 1) & (enfant_fip.rga.astype('int') == 6))
                )
            ].copy()
        enfant_fip['noifam'] = (100 * enfant_fip.ident + enfant_fip.noidec).astype(int)
        enfant_fip['famille'] = 50
        enfant_fip['kid'] = True
        enfant_fip['ident'] = None  # TODO: should we really do this ?
        assert_dtype(enfant_fip.kid, "bool")
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(enfant_fip[series_name], "int")
            except Exception:
                assert_dtype(enfant_fip[series_name], "int64")
        control_04(enfant_fip, base)
        famille = pd.concat([famille, enfant_fip])
        base = pd.concat([base, enfant_fip])
        parent_fip = famille[famille.noindiv.isin(enfant_fip.noifam.values)].copy()
        assert (enfant_fip.noifam.isin(parent_fip.noindiv.values)).any(), \
            "{} doublons entre enfant_fip et parent fip !".format(
                (enfant_fip.noifam.isin(parent_fip.noindiv.values)).sum())
        parent_fip['noifam'] = parent_fip['noindiv'].values.copy()
        parent_fip['famille'] = 51
        parent_fip['kid'] = False
        log.debug("Contrôle de parent_fip")
        control_04(parent_fip, base)
        control_04(famille, base)
        famille = famille.merge(parent_fip, how='outer')
        del enfant_fip, fip, parent_fip

    assert not famille.duplicated().any()

    # TODO: clean this mess
    # manually removing a family (which year?)
    famille = famille[famille["noifam"] != 1402071301]
    if year == 2013:
        famille = famille[famille["noindiv"] != 1300584605]
        famille = famille[famille["noindiv"] != 1302648201]
    elif year == 2015:
        famille = famille[famille["noindiv"] != 1501024703]
    else :
        pass # no such issue detected on other years
    assert not famille.noindiv.duplicated().any()
    control_04(famille, base)
    return base, famille


def famille_6(base = None, famille = None, personne_de_reference = None, kind = 'erfs_fpr', year = None):
    assert base is not None
    assert famille is not None
    assert personne_de_reference is not None
    assert year is not None
    log.info(" [6] gestion des non attribués")
    log.info(" [6.1] non attribués type 1")
    non_attribue1 = subset_base(base, famille)
    lien = 'lien' if year < 2013 else 'lienprm'  # TODO: attention pas les mêmes modalités
    subsetlienfamilial = list(range(1,5)) if year<2013 else list(range(6))
    agepr = 'agepr' if year < 2013 else "ageprm"
    if kind == 'erfs_fpr':
        non_attribue1 = non_attribue1[
            non_attribue1.moins_de_15_ans_inclus |
            (
                non_attribue1.jeune_non_eligible_rsa &
                (non_attribue1[lien].isin(subsetlienfamilial)) &
                (non_attribue1[agepr] >= 35)
                )
            ].copy()

    else:
        non_attribue1 = non_attribue1[
            (~(non_attribue1.quelfic != 'FIP')) &
            (
                non_attribue1.moins_de_15_ans_inclus |
                (
                    non_attribue1.jeune_non_eligible_rsa &
                    (non_attribue1[lien].isin(subsetlienfamilial)) &
                    (non_attribue1[agepr]>= 35)
                    )
                )
            ].copy()
    # On rattache les moins de 15 ans avec la PR (on a déjà éliminé les enfants en nourrice)
    log.debug("Il y a {} enfants non attribués de type 1".format(
        len(non_attribue1.index)))

    if len(non_attribue1.index) > 0:
        non_attribue1 = non_attribue1.merge(personne_de_reference)
        non_attribue1['famille'] = 61 * non_attribue1.moins_de_15_ans_inclus + 62 * ~(non_attribue1.moins_de_15_ans_inclus)
        non_attribue1['kid'] = True
        assert_dtype(non_attribue1.kid, "bool")
        try:
            assert_dtype(non_attribue1.famille, "int")
        except Exception:
            assert_dtype(non_attribue1.famille, "int64")
        famille = pd.concat([famille, non_attribue1], sort = True)
        control_04(famille, base)
        del personne_de_reference, non_attribue1

    log.info(" [6.2] non attribué type 2")
    if kind == 'erfs_fpr':
        non_attribue2 = base[~(base.noindiv.isin(famille.noindiv.values))].copy()
    else:
        non_attribue2 = base[(~(base.noindiv.isin(famille.noindiv.values)) & (base.quelfic != "FIP"))].copy()

    log.debug("Il y a {} enfants non attribués de type 2".format(
        len(non_attribue2.index)))
    if len(non_attribue2.index) > 0:
        non_attribue2['noifam'] = (100 * non_attribue2.ident + non_attribue2.noi).astype(int)
        non_attribue2['kid'] = False
        non_attribue2['famille'] = 63
        assert_dtype(non_attribue2.kid, "bool")
        for series_name in ['famille', 'noifam']:
            try:
                assert_dtype(non_attribue2[series_name], "int")
            except Exception:
                assert_dtype(non_attribue2[series_name], "int64")
        famille = pd.concat([famille, non_attribue2])
        control_04(famille, base)
        del non_attribue2

    return base, famille


# helpers

def control_04(dataframe, base):
    log.debug("longueur de la dataframe après opération : {}".format(len(dataframe)))

    if any(dataframe.duplicated(subset = 'noindiv')):
        log.debug("contrôle des doublons : il y a {} individus en double".format(
            dataframe.duplicated(subset = 'noindiv').sum()))
        # dataframe[dataframe.noindiv.duplicated()].to_csv("laoslasiecoule.csv")
        # lllt = dataframe[dataframe.noindiv.duplicated()]["noindiv"]
        # dataframe[dataframe.noindiv.isin(lllt)].to_csv("bronskibeat.csv")
    # log.info("contrôle des colonnes : il y a {} colonnes".format(len(dataframe.columns)))
    log.debug("Il y a {} identifiants de familles différentes".format(len(dataframe.noifam.unique())))
    assert not dataframe.noifam.isnull().any(), "{} noifam are NaN".format(dataframe.noifam.isnull().sum())

    log.debug("{} lignes dans dataframe vs {} lignes dans base".format(len(dataframe.index), len(base.index)))
    assert len(dataframe.index) <= len(base.index), "dataframe has too many rows compared to base"
    assert set(dataframe.noifam.unique()).issubset(set(base.noindiv)), \
        "The following {} are not in the dataframe: \n {}".format(
            len(dataframe.loc[~dataframe.noifam.isin(base.noindiv)]),
            dataframe.loc[~dataframe.noifam.isin(base.noindiv), ['noifam', 'famille']],
            )

    if 'quifam' in dataframe:
        famille_population = dataframe.query('quifam == 0').groupby('famille')['wprm'].sum()
        log.debug("famille :\n{}".format(
            famille_population / famille_population.sum()
            ))


def famille_7(base = None, famille = None, indivi = None, kind = 'erfs_fpr',
        skip_enfants_a_naitre = True, year = None):
    assert base is not None
    assert famille is not None
    assert indivi is not None
    assert year is not None
    log.info(" [7] Sauvegarde de la table famille")

    log.info(" [7.1] Mise en forme finale")
    famille['chef'] = (famille.noifam == (100 * famille.ident + famille.noi))
    assert_dtype(famille.chef, "bool")

    famille.reset_index(inplace = True)
    control_04(famille, base)

    log.info(" [7.2] création de la colonne rang")
    famille['rang'] = famille.kid.astype(int)
    while any(famille.loc[famille.rang != 0].duplicated(subset = ['rang', 'noifam'])):
        famille.loc[famille.rang != 0, 'rang'] += (
            famille[
                famille.rang != 0
                ]
            .duplicated(subset = ["rang", 'noifam']).values
            )

    log.info(" [7.3] création de la colonne quifam et troncature")
    log.debug("value_counts chef : \n {}".format(famille['chef'].value_counts()))
    log.debug("value_counts kid :' \n {}".format(famille['kid'].value_counts()))

    famille['quifam'] = -1
    famille['quifam'] = famille['quifam'].where(famille['chef'].values, 0)
    famille.quifam = (
        0 +
        ((~famille.chef) & (~famille.kid)).astype(int) +
        famille.kid * (famille.rang + 1)
        ).astype('int')

    assert (famille.groupby('noifam')['chef'].sum() <= 1).all(), "Il y a plusieurs chefs par famille"

    if not (famille.groupby('noifam')['chef'].sum() == 1).all():
        log.debug("Il y a {} familles qui n'ont pas de chef de famille".format(
            (famille.groupby('noifam')['chef'].sum() == 0).sum()
            ))
        absence_chef = (famille.groupby('noifam')['chef'].sum() == 0)
        noifam_absence_chef = absence_chef.loc[absence_chef].index
        idents = famille.loc[famille.noifam.isin(noifam_absence_chef), 'ident'].unique()
        log.debug(u'Il y a {} ménages contenant des familles sans chefs. on les retire'.format(
            len(idents)))
        famille = famille.loc[~famille.ident.isin(idents)].copy()
        indivi = indivi.loc[~indivi.ident.isin(idents)].copy()

        control_04(famille, base)

    log.info(u"value_counts quifam : \n {}".format(famille['quifam'].value_counts().sort_index()))

    #TODO: pourquoi une famille en 2006 a deux personnes marquees comme conjoint?
    #famille["dup"] = famille.duplicated(subset = ['noifam', 'quifam'], keep = False)
    dup = famille.duplicated(subset = ['noifam', 'quifam'], keep = False)

    for fam in famille.loc[dup==True,"noifam"]:   # ici fam = 601882501
        famille.loc[(famille.noifam==fam) & (famille.quifam != 0),
                "quifam"] = famille.loc[(famille.noifam==fam) & (famille.quifam != 0),
                                        "age"].copy().rank(ascending=False).astype('int')

    famille = famille[['noindiv', 'quifam', 'noifam']].copy()
    famille.rename(columns = {'noifam': 'idfam'}, inplace = True)
    log.debug("Vérifications sur famille")

    log.info(u"Vérifications sur famille")
    duplicated_famillle = famille.duplicated(subset = ['idfam', 'quifam'], keep = False)
    if duplicated_famillle.sum() > 0:
        log.warning("There are {} duplicates of quifam inside famille [{}]".format(
            duplicated_famillle.sum(), duplicated_famillle))
        # raise

    individus = indivi.merge(famille, on = ['noindiv'], how = "inner")
    if skip_enfants_a_naitre:
        del indivi, base
    else:
        del indivi, enfants_a_naitre, base
    gc.collect()

    return individus


def subset_base(base, famille):
    """
    Generates a dataframe containing the values of base that are not already in famille
    """
    return base.loc[~(base.noindiv.isin(famille.noindiv.values))].copy()


if __name__ == '__main__':
    import sys
    logging.basicConfig(level = logging.INFO, stream = sys.stdout)
    # logging.basicConfig(level = logging.INFO, filename = 'step_04.log', filemode = 'w')
    year = 2014
    build_famille(year = year)



#! /usr/bin/env python
import logging
import os



from openfisca_france_data.erfs.input_data_builder import (  # analysis:ignore
    step_01_pre_processing as pre_processing,
    step_02_imputation_loyer as imputation_loyer,
    step_03_fip as fip,
    step_04_famille as famille,
    step_05_foyer as foyer,
    step_06_rebuild as rebuild,
    step_07_invalides as invalides,
    step_08_final as final,
    )
from openfisca_survey_manager.temporary import get_store

from openfisca_survey_manager.surveys import Survey
from openfisca_survey_manager.survey_collections import SurveyCollection


log = logging.getLogger(__name__)


def run_all(year = None, check = False):

    assert year is not None
    pre_processing.create_indivim_menagem(year = year)
    pre_processing.create_enfants_a_naitre(year = year)
    #    try:
    #        imputation_loyer.imputation_loyer(year = year)
    #    except Exception, e:
    #        log.info('Do not impute loyer because of the following error: \n {}'.format(e))
    #        pass
    fip.create_fip(year = year)
    famille.famille(year = year)
    foyer.sif(year = year)
    foyer.foyer_all(year = year)
    rebuild.create_totals_first_pass(year = year)
    rebuild.create_totals_second_pass(year = year)
    rebuild.create_final(year = year)
    invalides.invalide(year = year)
    final.final(year = year, check = check)

    temporary_store = get_store(file_name = 'erfs')
    data_frame = temporary_store['input_{}'.format(year)]
    # Saving the data_frame
    openfisca_survey_collection = SurveyCollection(name = "openfisca", config_files_directory = config_files_directory)
    output_data_directory = openfisca_survey_collection.config.get('data', 'output_directory')
    survey_name = "openfisca_data_{}".format(year)
    table = "input"
    hdf5_file_path = os.path.join(os.path.dirname(output_data_directory), "{}.h5".format(survey_name))
    survey = Survey(
        name = survey_name,
        hdf5_file_path = hdf5_file_path,
        )
    survey.insert_table(name = table, data_frame = data_frame)
    openfisca_survey_collection.surveys.append(survey)
    collections_directory = openfisca_survey_collection.config.get('collections', 'collections_directory')
    json_file_path = os.path.join(collections_directory, 'openfisca.json')
    openfisca_survey_collection.dump(json_file_path = json_file_path)


if __name__ == '__main__':
    import time
    start = time.time()
    logging.basicConfig(level = logging.INFO, filename = 'run_all.log', filemode = 'w')
    run_all(year = 2009, check = False)
    log.info("Script finished after {}".format(time.time() - start))



import pytest

from openfisca_france_data.erfs_fpr.get_survey_scenario import get_survey_scenario
from openfisca_france_data import base_survey


def test_inflation():
    period = year = 2013
    survey_scenario = get_survey_scenario(
        year = year,
        tax_benefit_system = base_survey.france_data_tax_benefit_system
        )
    target_by_variable = dict(
        salaire_imposable = 1.2e08,
        )
    survey_scenario.inflate(target_by_variable = target_by_variable, period = period)
    assert abs(survey_scenario.compute_aggregate('salaire_imposable', period = period) - 1.2e08) / 1.2e08 < 1e-6, \
        "{} != {}".format(survey_scenario.compute_aggregate('salaire_imposable', period = period), 1.2e08)



from openfisca_france_data.erfs_fpr.get_survey_scenario import get_survey_scenario
from openfisca_france_data.reforms.inversion_directe_salaires import inversion_directe_salaires

survey_scenario = get_survey_scenario(year = 2014, reform = inversion_directe_salaires)


def test_inversion():
    survey_scenario.summarize_variable('salaire_de_base', force_compute = True)

    survey_scenario.summarize_variable('salaire_imposable', force_compute = True)
    survey_scenario.summarize_variable('salaire_de_base', force_compute = True)

    survey_scenario.summarize_variable('csg_imposable_salaire')
    data_frame_by_entity = survey_scenario.create_data_frame_by_entity(
        variables = [
            'salaire_de_base',
            'salaire_imposable',
            # 'csg_imposable_salaire',
            'categorie_salarie',
            'salaire_imposable_pour_inversion',
            # 'weight_individus',
            # 'wprm',
            ],
        )

    #famille = data_frame_by_entity['famille']
    individu = data_frame_by_entity['individu']
    menage = data_frame_by_entity['menage']

    individu.categorie_salarie.value_counts()
    prive = individu.query('categorie_salarie in [0, 1]')
    (prive.salaire_imposable - prive.salaire_imposable_pour_inversion).abs().max()
    survey_scenario.summarize_variable('salaire_imposable_pour_inversion')
    survey_scenario.summarize_variable('csg_deductible_salaire')
    survey_scenario.summarize_variable('chomage_brut')


if __name__ == '__main__':
    test_inversion()



import inspect
import logging
import os
import pkg_resources
import pandas

from openfisca_core import reforms  # type: ignore

import openfisca_france  # type: ignore

# Load input variables and output variables into entities
from openfisca_france_data.model import common, survey_variables  # noqa analysis:ignore
from openfisca_france_data.model.base import * # noqa  analysis:ignore


log = logging.getLogger(__name__)


def get_variables_from_module(module):
    variables = [
        getattr(module, item)
        for item in dir(module)
        if inspect.isclass(getattr(module, item))
        ]
    return [
        variable for variable in variables if issubclass(variable, Variable)  # noqa  analysis:ignore
        ]


def get_variables_from_modules(modules):
    variables = list()
    for module in modules:
        variables.extend(get_variables_from_module(module))
    return variables


def impute_take_up(target_probability, eligible, weights, recourant_last_period, seed):
    """
    Compute a vector of boolean for take_up according a target probability accross eligble population
    """
    assert (target_probability >= 0) and (target_probability <= 1)
    if target_probability == 0:
        return eligible * False
    elif target_probability == 1:
        return eligible * True

    data = pandas.DataFrame({
        'eligible': eligible,
        'weights': weights,
        'deja_recourant': recourant_last_period,
        })

    eligibles = data.loc[data.eligible == 1].copy()
    eligibles['recourant'] = eligibles.deja_recourant
    assert len(eligibles) > 0, "Aucun eligibles : impossible d'imputer le non recours"
    taux_recours_provisoire = eligibles.loc[eligibles.recourant].weights.sum() / eligibles.weights.sum()

    if target_probability > taux_recours_provisoire:
        adjusted_target_probability = (target_probability - taux_recours_provisoire) / (1 - taux_recours_provisoire)
        s_data = eligibles.loc[~eligibles.recourant].copy()
        s_data = s_data.sample(frac = adjusted_target_probability, replace = False, axis = 0, random_state = seed)
        eligibles.loc[eligibles.index.isin(s_data.index), 'recourant'] = True
        eligibles_recourants_indices = eligibles.query('eligible & recourant').index
        data['recourant'] = False
        data.loc[data.index.isin(eligibles_recourants_indices), 'recourant'] = True

    elif target_probability <= taux_recours_provisoire:
        adjusted_target_probability = 1 - target_probability / taux_recours_provisoire
        s_data = eligibles.loc[eligibles.recourant].copy()
        s_data = s_data.sample(frac = adjusted_target_probability, replace = False, axis = 0, random_state = seed)
        eligibles_non_recourant_indices = s_data.index
        data['recourant'] = data.deja_recourant & data.eligible
        data.loc[data.index.isin(eligibles_non_recourant_indices), 'recourant'] = False

    return data.recourant.values


def select_to_match_target(target_probability = None, target_mass = None, eligible = None, weights = None, take = None, seed = None):
    """
    Compute a vector of boolean for take_up according a target probability accross eligble population.
    If there is no eligible population, it returns False for the whole population.
    """
    if not(eligible.any()):
        print("No eligible observations : none of the observations will be selected. The target probability is not matched.")
        return eligible
    assert (target_probability is not None) or (target_mass is not None)
    if take is None:
        take = False
    if target_mass is not None:
        assert target_mass <= (eligible * weights).sum(), "target too high {}, {}".format(
            target_mass, (eligible * weights).sum())
        target_probability = target_mass / (eligible * weights).sum()

    assert (target_probability >= 0) and (target_probability <= 1)

    if target_probability == 0:
        return eligible * False
    elif target_probability == 1:
        return eligible * True

    data = pandas.DataFrame({
        'eligible': eligible,
        'weights': weights,
        'take': take,
        }).copy()

    eligibles = data.loc[data.eligible].copy()
    eligibles['selected'] = eligibles['take'].copy()
    initial_probability = eligibles.loc[eligibles['take'], 'weights'].sum() / eligibles.weights.sum()

    if target_probability > initial_probability:
        adjusted_target_probability = (target_probability - initial_probability) / (1 - initial_probability)
        s_data = eligibles.loc[~eligibles.selected].copy()
        s_data = s_data.sample(frac = adjusted_target_probability, replace = False, axis = 0, random_state = seed)
        eligibles.loc[eligibles.index.isin(s_data.index), 'selected'] = True
        eligibles_selected_indices = eligibles.query('eligible & selected').index
        data['selected'] = False
        data.loc[data.index.isin(eligibles_selected_indices), 'selected'] = True

    elif target_probability <= initial_probability:
        print("The target probability set ({}) is too low compared to the number of eligibles to take. Some individuals with take == True will be unselected to match the target probability.".format(target_probability))
        adjusted_target_probability = 1 - target_probability / initial_probability
        s_data = eligibles.loc[eligibles.selected].copy()
        s_data = s_data.sample(frac = adjusted_target_probability, replace = False, axis = 0, random_state = seed)
        eligibles_unselected_indices = s_data.index
        data['selected'] = data['take'] & data.eligible
        data.loc[data.index.isin(eligibles_unselected_indices), 'selected'] = False

    return data.selected.values



variables = get_variables_from_modules([common, survey_variables])


class openfisca_france_data(reforms.Reform):

    def apply(self):
        for variable in variables:
            if variable == Variable:
                continue
            try:
                self.add_variable(variable)
            except AttributeError:
                self.update_variable(variable)


openfisca_france_tax_benefit_system = openfisca_france.FranceTaxBenefitSystem()
# The existence of either CountryTaxBenefitSystem or country_tax_benefit_system is mandatory
france_data_tax_benefit_system = openfisca_france_data(openfisca_france_tax_benefit_system)

CountryTaxBenefitSystem = lambda: france_data_tax_benefit_system  # noqa analysis:ignore

AGGREGATES_DEFAULT_VARS = [
    "cotisation_salariales",
    #"cotisations_employeur",
    "cotisation_non_salarie",
    "salaire_brut",
    "retraite_brute",
    "chomage_brut",
    "salaire_imposable",
    "retraite_imposable",
    "chomage_imposable",
    "csg_salaire",
    "csg_non_salarie",
    "csg_remplacement",
    "csg_revenus_capital",
    'csg',
    'crds',
    'impot_revenu',
    'irpp',
    'prelevement_forfaitaire_unique_ir',
    'prelevement_forfaitaire_liberatoire',
    'taxe_habitation',
    #'ppe',
    #'ppe_brute',
    'af',
    'af_base',
    'af_majoration',
    'af_allocation_forfaitaire',
    'cf',
    'paje_base',
    'paje_naissance',
    'paje_prepare',
    'paje_cmg',
    'ars',
    #'aeeh',
    'asf',
    'aspa',
    'aah',
    'caah',
    'rsa',
    'ppa',
    #'aefa',
    #'api',
    #'psa',
    'aides_logement',
    'alf',
    'als',
    'apl',
    #'garantie_jeunes'
    ]
#  ajouter csgd pour le calcul des agrégats erfs
#  ajouter rmi pour le calcul des agrégats erfs


COUNTRY_DIR = os.path.dirname(os.path.abspath(__file__))
DATA_DIR = os.path.join(
    pkg_resources.get_distribution('openfisca-france-data').location,
    'openfisca_france_data',
    'plugins',
    'aggregates',
    )



from itertools import izip

from numpy import arange, array, floor, where

from openfisca_france_data.model.base import *  # noqa


class nbinde(Variable):
    value_type = int
    entity = Menage
    label = "Nombre d'individus dans le ménage. La valeur varie entre 1 et 6 pour 6 membres et plus."
    definition_period = YEAR

    def formula(menage, period, parameters):
        """
        Number of household members
        """

        age_en_mois_i = menage.members('age_en_mois', period)

        n1 = menage.sum(floor(age_en_mois_i) >= 0)

        return where(n1 >= 6, 6, n1)


def _ageq(age_en_mois):
    '''
    Calcule la tranche d'âge quinquennal
    moins de 25 ans : 0
    25 à 29 ans     : 1
    30 à 34 ans     : 2
    35 à 39 ans     : 3
    40 à 44 ans     : 4
    45 à 49 ans     : 5
    50 à 54 ans     : 6
    55 à 59 ans     : 7
    60 à 64 ans     : 8
    65 à 69 ans     : 9
    70 à 74 ans     :10
    75 à 79 ans     :11
    80 ans et plus  :12
    'ind'
    '''
    age = floor(age_en_mois / 12)
    tranche = array([(age >= ag) for ag in arange(25, 5, 81)]).sum(axis = 0)
    return tranche


def _nb_ageq0(self, age_en_mois_holder):
    '''
    Calcule le nombre d'individus dans chaque tranche d'âge quinquennal (voir ageq)
    'men'
    '''
    age_en_mois = self.split_by_roles(age_en_mois_holder)

    ag1 = 0
    nb = 0
    for agm in age_en_mois.itervalues():
        age = floor(agm / 12)
        nb += (ag1 <= age) & (age <= (ag1 + 4))
    return nb


class cohab(Variable):
    value_type = bool
    entity = Menage
    label = "Vie en couple"
    definition_period = YEAR

    def formula(menage, period, parameters):
        '''
        Indicatrice de vie en couple
        'men'
        '''

        return menage.nb_persons(role = Menage.CONJOINT) == 1


class act_cpl(Variable):
    is_period_size_independent = True
    value_type = int
    entity = Menage
    label = "Nombre d'actifs parmi la personne de référence du méange et son conjoint"
    definition_period = YEAR

    def formula(menage, period, parameters):
        '''
        Nombre d'actifs parmi la personne de référence et son conjoint
        'men'
        '''
        cohab = menage('cohab', period)

        return (
            1 * (menage.personne_de_reference('activite', period) <= 1) +
            1 * (menage.conjoint('activite', period) <= 1)
            ) * cohab


class act_enf(Variable):
    is_period_size_independent = True
    value_type = int
    entity = Menage
    label = "Nombre d'enfants actifs"
    definition_period = YEAR

    def formula(menage, period, parameters):
        '''
        Nombre de membres actifs du ménage autre que la personne de référence ou son conjoint
        'men'
        '''
        act_i = menage.members('activite', period)

        return menage.sum(1 * (act_i <= 1), role = Menage.ENFANT)


def _nb_act(act_cpl, act_enf):
    '''
    Nombre de membres actifs du ménage
    'men'
    '''
    return act_cpl + act_enf


# def _cplx(typmen15):
class cplx(Variable):
    value_type = bool
    entity = Menage
    label = "Indicatrice de ménage complexe. Un ménage est complexe si les personnes autres que la personne de référence ou son conjoint ne sont pas enfants."
    definition_period = YEAR

    def formula(menage, period, parameters):
        """
        Indicatrice de ménage complexe

        Un ménage est complexe si les personnes autres que la personne de référence ou son conjoint ne sont pas enfants.
        """

        individus = menage.members
        age_i = individus('age', period)

        is_enfant_plus_25 = individus.has_role(Famille.ENFANT) * (age_i > 25)
        is_not_pref = not_(individus.has_role(Menage.PERSONNE_DE_REFERENCE))
        is_demandeur = individus.has_role(Famille.DEMANDEUR)

        return menage.any(
            is_not_pref * is_demandeur + is_enfant_plus_25
            )



    # En fait on ne peut pas car on n'a les enfants qu'au sens des allocations familiales ...
    # return (typmen15 > 12)


class TypesMenage15(Enum):
    __order__ = 'inconnu personne_seule_active personne_seule_inactive famille_monoparentale_parent_actif famille_monoparentale_parent_inactif_enfant_actif famille_monoparentale_inactifs couple_sans_enfants_1_actif couple_sans_enfants_2_actifs couple_sans_enfants_inactifs couple_avec_enfants_1_actif couple_avec_enfants_2_actifs couple_avec_enfants_parents_inactifs_enfant_actif couple_avec_enfants_inactifs autres_1_actif autres_2_actifs autres_inactifs'  # Needed to preserve the order in Python 2
    inconnu = ""
    personne_seule_active = "Personne seule active"
    personne_seule_inactive = "Personne seule inactive"
    famille_monoparentale_parent_actif = "Familles monoparentales, parent actif"
    famille_monoparentale_parent_inactif_enfant_actif = "Familles monoparentales, parent inactif et au moins un enfant actif"
    famille_monoparentale_inactifs = "Familles monoparentales, tous inactifs"
    couple_sans_enfants_1_actif = "Couples sans enfant, 1 actif"
    couple_sans_enfants_2_actifs = "Couples sans enfant, 2 actifs"
    couple_sans_enfants_inactifs = "Couples sans enfant, tous inactifs"
    couple_avec_enfants_1_actif = "Couples avec enfant, 1 membre du couple actif"
    couple_avec_enfants_2_actifs = "Couples avec enfant, 2 membres du couple actif"
    couple_avec_enfants_parents_inactifs_enfant_actif = "Couples avec enfant, couple inactif et au moins un enfant actif"
    couple_avec_enfants_inactifs = "Couples avec enfant, tous inactifs"
    autres_1_actif = "Autres ménages, 1 actif"
    autres_2_actifs = "Autres ménages, 2 actifs ou plus"
    autres_inactifs = "Autres ménages, tous inactifs"


class typmen15(Variable):
    value_type = Enum
    possible_values = TypesMenage15
    default_value = TypesMenage15.inconnu
    entity = Menage
    label = "Type de ménage"
    definition_period = YEAR

    def formula(menage, period, parameters):
        '''
        Type de ménage en 15 modalités
        1 Personne seule active
        2 Personne seule inactive
        3 Familles monoparentales, parent actif
        4 Familles monoparentales, parent inactif et au moins un enfant actif
        5 Familles monoparentales, tous inactifs
        6 Couples sans enfant, 1 actif
        7 Couples sans enfant, 2 actifs
        8 Couples sans enfant, tous inactifs
        9 Couples avec enfant, 1 membre du couple actif
        10 Couples avec enfant, 2 membres du couple actif
        11 Couples avec enfant, couple inactif et au moins un enfant actif
        12 Couples avec enfant, tous inactifs
        13 Autres ménages, 1 actif
        14 Autres ménages, 2 actifs ou plus
        15 Autres ménages, tous inactifs
        'men'
        '''
        nbinde = menage('nbinde', period)
        cohab = menage('cohab', period)
        act_cpl = menage('act_cpl', period)
        cplx = menage('cplx', period)
        act_enf = menage('act_enf', period)

        res = 0 + (cplx == 0) * (
                1 * ((nbinde == 1) & (cohab == 0) & (act_cpl == 1)) +  # Personne seule active
                2 * ((nbinde == 1) & (cohab == 0) & (act_cpl == 0)) +  # Personne seule inactive
                3 * ((nbinde > 1) & (cohab == 0) & (act_cpl == 1)) +  # Familles monoparentales, parent actif
                4 * ((nbinde > 1) & (cohab == 0) & (act_cpl == 0) & (act_enf >= 1)) +  # Familles monoparentales, parent inactif et au moins un enfant actif
                5 * ((nbinde > 1) & (cohab == 0) & (act_cpl == 0) & (act_enf == 0)) +  # Familles monoparentales, tous inactifs
                6 * ((nbinde == 2) & (cohab == 1) & (act_cpl == 1)) +  # Couples sans enfant, 1 actif
                7 * ((nbinde == 2) & (cohab == 1) & (act_cpl == 2)) +  # Couples sans enfant, 2 actifs
                8 * ((nbinde == 2) & (cohab == 1) & (act_cpl == 0)) +  # Couples sans enfant, tous inactifs
                9 * ((nbinde > 2) & (cohab == 1) & (act_cpl == 1)) +  # Couples avec enfant, 1 membre du couple actif
                10 * ((nbinde > 2) & (cohab == 1) & (act_cpl == 2)) +  # Couples avec enfant, 2 membres du couple actif
                11 * ((nbinde > 2) & (cohab == 1) & (act_cpl == 0) & (act_enf >= 1)) +  # Couples avec enfant, couple inactif et au moins un enfant actif
                12 * ((nbinde > 2) & (cohab == 1) & (act_cpl == 0) & (act_enf == 0))  # Couples avec enfant, tous inactifs
                               ) + (cplx == 1) * (
                13 * (((act_cpl + act_enf) == 1)) +  # Autres ménages, 1 actif
                14 * (((act_cpl + act_enf) > 1)) +  # Autres ménages, 2 actifs ou plus
                15 * (((act_cpl + act_enf) == 0)))  # Autres ménages, tous inactifs

        # ratio = (( (typmen15!=res)).sum())/((typmen15!=0).sum())
        # ratio  2.7 % d'erreurs enfant non nés et erreur d'enfants
        return res



def year_specific_by_generic_data_frame_name(year):
    yr = str(year)[2:]
    yr1 = str(year + 1)[2:]
    return dict(
        # Enquête revenu fiscaux, table ménage
        erf_menage = "menage" + yr,
        # Enquête emploi en continu, table ménage
        eec_menage = "mrf" + yr + "e" + yr + "t4",
        # Enquête revenu fiscaux, table foyer # compressed file for 2006
        foyer = "foyer" + yr,
        # Enquête revenu fiscaux, table individu
        erf_indivi = "indivi{}".format(yr),
        # Enquête emploi en continue, table individu
        eec_indivi = "irf" + yr + "e" + yr + "t4",
        # Enquête emploi en continue, tables complémentaires 1
        eec_cmp_1 = "icomprf" + yr + "e" + yr1 + "t1",
        eec_cmp_2 = "icomprf" + yr + "e" + yr1 + "t2",
        eec_cmp_3 = "icomprf" + yr + "e" + yr1 + "t3",
        )



import click
import ipdb as pdb
import logging
import pandas as pd
import sys
import yaml

from pandas.api.types import is_datetime64_any_dtype as is_datetime
from openfisca_core.model_api import ETERNITY


from openfisca_france_data import france_data_tax_benefit_system
from openfisca_france_data.erfs_fpr import original_id_by_entity
from openfisca_france_data.erfs_fpr.scenario import ErfsFprSurveyScenario
from openfisca_france_data.erfs_fpr.comparison import ErfsFprtoInputComparator
from openfisca_france_data.erfs_fpr.get_survey_scenario import variables_converted_to_annual


tax_benefit_system = france_data_tax_benefit_system
openfisca_variables_by_entity = dict(
    (
        entity.key,
        [variable_name for variable_name, variable in tax_benefit_system.variables.items() if variable.entity.key == entity.key],
        )
    for entity in tax_benefit_system.entities
    )

id_variable_by_entity_key = ErfsFprSurveyScenario.id_variable_by_entity_key
weight_variable_by_entity = ErfsFprSurveyScenario.weight_variable_by_entity


renaming_variables_to_annual = {
    monthly_variable: f"{monthly_variable}_annuel"
    for monthly_variable in variables_converted_to_annual
    }

def build_test(period, noindiv, target_variables = None):
    if target_variables is None:
        target_variables = ErfsFprtoInputComparator.default_target_variables

    comparator = ErfsFprtoInputComparator()
    comparator.period = period
    input_dataframe_by_entity, target_dataframe_by_entity = comparator.get_test_dataframes(rebuild = True, noindivs = [noindiv])

    def convert_date_to_sting(dataframe):
        date_columns = list(dataframe.select_dtypes(include=["datetime64"]))
        dataframe[date_columns] = dataframe[date_columns].astype(str)

    def remove_non_openfisca_columns(dataframe):
        openfisca_variables = set(sum([list(value) for value in openfisca_variables_by_entity.values()], [])).union(set(["noindiv", "idmen_original"]))
        selected_variables = list(set(dataframe.columns).intersection(openfisca_variables))
        return dataframe[selected_variables]

    def build_test_dict(dataframe_by_entity, renaming_variables_to_annual = None):
        input_by_entity = dict()
        for entity, dataframe in dataframe_by_entity.items():
            convert_date_to_sting(dataframe)
            identifier = "noindiv" if entity == "individu" else "idmen_original"
            entity_plural = "individus" if entity == "individu" else "menages"
            input_by_entity[entity_plural] = input = dict()
            dataframe[identifier] = "id_" + dataframe[identifier].astype(str)
            df = remove_non_openfisca_columns(dataframe).set_index(identifier)
            for row, series in df.iterrows():
                series.drop(
                    (
                        list(weight_variable_by_entity.values())
                        + list(id_variable_by_entity_key.values())
                        + list(original_id_by_entity.values())
                        ),
                    inplace = True,
                    errors = "ignore",
                    )
                if renaming_variables_to_annual:
                    series.rename(renaming_variables_to_annual, inplace = True)
                input[row] = series.dropna().to_dict()

        return input_by_entity

    input_by_entity = build_test_dict(input_dataframe_by_entity)
    output_by_entity = build_test_dict(target_dataframe_by_entity, renaming_variables_to_annual)

    relative_error_margin = {
        "default": 5e-3,
        }
    test = dict(
        name = f"Observation {noindiv} on {period}",
        reforms = "openfisca_france_data.erfs_fpr.get_survey_scenario.erfs_fpr_plugin",
        max_spiral_loops = 4,
        relative_error_margin = relative_error_margin,
        period = period,
        input = input_by_entity,
        output = output_by_entity,
        )
    return test



def export_test_file(period, noindiv, test_case_name = None):
    """
    Export a erfs-fpr input and output to an OpenFisca test case.

    Args:
        period (int): simulation year
        noindiv (int): individu id number
        test_case_name (str, optional): _description_. Defaults to Name of the test case file. Defaults to 'test_case_erfs_fpr_NOINDIV'.
    """
    if test_case_name is None:
        test_case_name = f"test_case_erfs_fpr_{noindiv}"

    test_case_file_path = f'{test_case_name}.yaml'
    test = build_test(period, noindiv)

    with open(test_case_file_path, 'w') as file:
        yaml.dump(test, file, sort_keys=False)

    text = get_erfs_fpr_data_as_comment(noindiv)

    with open(test_case_file_path, "a+") as file:
        _ = file.read()  # this auto closes the file after reading, which is a good practice
        file.write(text)


def get_erfs_fpr_data_as_comment(noind):
    return "# Blabal"


@click.command()
@click.option('-n', '--noindiv', type = int, help = "Individual id number", required = True)
@click.option('-v', '--verbose', is_flag = True, default = False, help = "Increase output verbosity", show_default = True)
@click.option('-d', '--debug', is_flag = True, default = False, help = "Use python debugger", show_default = True)
def create_test(noindiv = 0, verbose = False, debug = False):
    """Create test case for a specific ERFS FPR individual."""
    assert noindiv != 0, "Provide valid individual"
    logging.basicConfig(level = logging.DEBUG if verbose else logging.WARNING, stream = sys.stdout)
    from openfisca_france_data.erfs_fpr import REFERENCE_YEAR
    period = REFERENCE_YEAR
    try:
        export_test_file(period, noindiv)
    except Exception as e:
        if debug:
            pdb.post_mortem(sys.exc_info()[2])
        raise e


if __name__ == "__main__":
    sys.exit(create_test())



import pytest

from openfisca_france_data.erfs_fpr.scenario import ErfsFprSurveyScenario


@pytest.fixture
def year() -> int:
    return 2019


def test_create_scenario(year):
    assert ErfsFprSurveyScenario(year)



#! /usr/bin/env python2
import logging


from openfisca_core import periods
from openfisca_france import FranceTaxBenefitSystem
from openfisca_france.model.base import TypesCategorieSalarie
from openfisca_france_data.erfs_fpr.input_data_builder.step_03_variables_individuelles import (
    create_ages,
    create_date_naissance,
    create_activite,
    create_revenus,
    create_contrat_de_travail,
    create_categorie_salarie,
    create_salaire_de_base,
    create_effectif_entreprise,
    create_traitement_indiciaire_brut,
    )
from openfisca_survey_manager.temporary import get_store

from openfisca_france_data.erfs_fpr.get_survey_scenario import get_survey_scenario


log = logging.getLogger(__name__)


variables_de_base = [
    'categorie_salarie',
    'contrat_de_travail',
    'heures_remunerees_volume',
    'hsup',
    'nombre_jours_calendaires',
    'salaire_imposable',
    'salaire_net',
    ]

cotisations_nulles_hors_prive = [
    # non contributives
    'agff_salarie',
    'agirc_salarie',
    'agirc_gmp_salarie',
    'apec_salarie',
    'arrco_salarie',
    'chomage_salarie',
    'cotisation_exceptionnelle_temporaire_salarie',
    'vieillesse_deplafonnee_salarie',
    'vieillesse_plafonnee_salarie',
    # contributives
    'mmid_salarie',
    ]

variables_nulles_hors_prive = [
    'complementaire_sante_salarie',
    'indemnite_fin_contrat',
    'salaire_de_base',
    ] + cotisations_nulles_hors_prive

cotisations_nulles_hors_fonction_publique = [
    # non contributives
    'ircantec_salarie',
    'pension_civile_salarie',
    'rafp_salarie',
    # contributives
    'contribution_exceptionnelle_solidarite',
    ]

variables_nulles_hors_fonction_publique = [
    'traitement_indiciaire_brut',
    'remuneration_principale',
    'primes_fonction_publique',
    'indemnite_residence',
    'supplement_familial_traitement',
    ] + cotisations_nulles_hors_fonction_publique


categories_salarie_du_public = [
    'public_titulaire_etat',
    'public_titulaire_hospitaliere',
    'public_titulaire_territoriale',
    'public_titulaire_militaire'
    ]

index_by_categorie_salarie_du_public = dict(
    (categorie, TypesCategorieSalarie[categorie].index)
    for categorie in categories_salarie_du_public
    )

index_by_categorie_salarie_hors_public = dict(
    (categorie, TypesCategorieSalarie[categorie].index)
    for categorie in TypesCategorieSalarie.__members__.keys()
    if categorie not in categories_salarie_du_public
    )


def check_variable_nullity(data_frame, variable, categorie_salarie, categorie_salarie_index):
    extraction = data_frame.query(
        'categorie_salarie == @categorie_salarie_index'
        )
    if not (extraction[variable] < 1).all():
        print(
            "{} non nul categorie_salarie={} ({})".format(
            variable, categorie_salarie_index, categorie_salarie)
            )
        print(
            extraction.loc[~(extraction[variable] < 1), variable].head()
            )


def check_nullity_public_variables(data_frame):
    for variable in variables_nulles_hors_fonction_publique:
        for categorie_salarie, categorie_salarie_index in index_by_categorie_salarie_hors_public.items():
            print("{}, {}".format(variable, categorie_salarie))
            if variable == 'indemnite_residence' and categorie_salarie == 'public_non_titulaire':
                continue

            check_variable_nullity(data_frame, variable, categorie_salarie, categorie_salarie_index)


def check_nullity_private_variables(data_frame):
    for variable in variables_nulles_hors_prive:
        for categorie_salarie, categorie_salarie_index in index_by_categorie_salarie_du_public.items():
            print("{}, {}".format(variable, categorie_salarie))
            check_variable_nullity(data_frame, variable, categorie_salarie, categorie_salarie_index)


def remove_some_variables_after_check(data_frame):
    zero_variables = [
        'complementaire_sante_salarie',
        'hsup',
        'indemnite_fin_contrat',
        'indemnite_residence',
        'supplement_familial_traitement',
        ]

    for variable in zero_variables:
        assert (data_frame[variable] == 0).all(), "{} is not always = 0".format(variable)
        del data_frame[variable]

    assert (data_frame['nombre_jours_calendaires'] == 366).all()
    del data_frame['nombre_jours_calendaires']


def test_create_salaire_de_base(year, revenu_type = 'net'):
    """
    Test create_salaire_de_base avec données de l'enquête erfs_fpr
    """
    temporary_store = get_store(file_name = 'erfs_fpr')
    individu = temporary_store['individu_for_inversion_{}'.format(year)]

    if revenu_type == 'net':
        salaire_pour_inversion = individu.salaire_net.copy()
    elif revenu_type == 'imposable':
        salaire_pour_inversion = individu.salaire_imposable.copy()

    id_variables = ['idfoy', 'idmen', 'idfam']
    for id_variable in id_variables:
        individu[id_variable] = range(0, len(individu))

    position_variables = ['quifoy', 'quimen', 'quifam']
    for position_variable in position_variables:
        individu[position_variable] = 0

    data = dict(
        input_data_frame_by_entity = dict(
            individu = individu,
            )
        )
    survey_scenario = get_survey_scenario(
        year = year,
        data = data,
        )

    survey_scenario.tax_benefit_system.neutralize_variable('indemnite_residence')

    variables = set(
        variables_de_base
        + variables_nulles_hors_prive
        + variables_nulles_hors_fonction_publique
        )
    data_frame = survey_scenario.create_data_frame_by_entity(
        variables = variables, period = periods.period(year)
        )['individu']
    data_frame['salaire_pour_inversion'] = salaire_pour_inversion
    data_frame.rename(columns = {'salaire_{}'.format(revenu_type): 'salaire'}, inplace = True)

    return survey_scenario, data_frame


def create_individu_for_inversion(year, revenu_type = 'net'):
    assert revenu_type in ['net', 'imposable']
    assert year is not None

    # Using data produced by preprocessing.build_merged_dataframes
    temporary_store = get_store(file_name = 'erfs_fpr')
    individus = temporary_store['individus_{}_post_01'.format(year)]

    if revenu_type == 'net':
        old_by_new_variables = {
            'chomage_i': 'chomage_net',
            'pens_alim_recue_i': 'pensions_alimentaires_percues',
            'rag_i': 'rag_net',
            'retraites_i': 'retraite_nette',
            'ric_i': 'ric_net',
            'rnc_i': 'rnc_net',
            'salaires_i': 'salaire_net',
            }
    elif revenu_type == 'imposable':
        old_by_new_variables = {
            'chomage_i': 'chomage_imposable',
            'pens_alim_recue_i': 'pensions_alimentaires_percues',
            'rag_i': 'rag_net',
            'retraites_i': 'retraite_imposable',
            'ric_i': 'ric_net',
            'rnc_i': 'rnc_net',
            'salaires_i': 'salaire_imposable',
            }

    for variable in old_by_new_variables:
        assert variable in individus.columns.tolist(), "La variable {} n'est pas présente".format(variable)

    individus.rename(
        columns = old_by_new_variables,
        inplace = True,
        )

    created_variables = []
    create_ages(individus, year)
    created_variables.append('age')
    created_variables.append('age_en_mois')

    create_date_naissance(individus, age_variable = None, annee_naissance_variable = 'naia', mois_naissance = 'naim',
        year = year)
    created_variables.append('date_naissance')

    period = periods.period(year)
    # create_revenus(individus, revenu_type = revenu_type)
    # created_variables.append('taux_csg_remplacement')

    create_contrat_de_travail(individus, period = period, salaire_type = revenu_type)
    created_variables.append('contrat_de_travail')
    created_variables.append('heures_remunerees_volume')

    create_categorie_salarie(individus, period = period)
    created_variables.append('categorie_salarie')

    tax_benefit_system = FranceTaxBenefitSystem()
    create_salaire_de_base(individus, period = period, revenu_type = revenu_type, tax_benefit_system = tax_benefit_system)
    created_variables.append('salaire_de_base')

    create_effectif_entreprise(individus, period = period)
    created_variables.append('effectif_entreprise')

    create_traitement_indiciaire_brut(individus, period = period, revenu_type = revenu_type,
        tax_benefit_system = tax_benefit_system)
    created_variables.append('traitement_indiciaire_brut')
    created_variables.append('primes_fonction_publique')

    other_variables = ['salaire_{}'.format(revenu_type)]
    temporary_store['individu_for_inversion_{}'.format(year)] = individus[
        created_variables + other_variables]


if __name__ == '__main__':
    year = 2012
    revenu_type = 'imposable'

    create_individu_for_inversion(year, revenu_type = revenu_type)
    survey_scenario, data_frame = test_create_salaire_de_base(year, revenu_type = revenu_type)

    check_nullity_public_variables(data_frame)
    check_nullity_private_variables(data_frame)
    remove_some_variables_after_check(data_frame)

    data_frame['absolute_error'] =  (data_frame.salaire - data_frame.salaire_pour_inversion).abs()
    data_frame['relative_error'] = (
        (data_frame.salaire - data_frame.salaire_pour_inversion).abs()
        / (data_frame.salaire_pour_inversion + 1 * (data_frame.salaire_pour_inversion == 0))
        )

    absolute_error_threshold = 5
    relative_error_threshold = .001

    dispatch = ['categorie_salarie', 'contrat_de_travail']
    data_frame['absolute_errored'] = data_frame['absolute_error'] > absolute_error_threshold
    data_frame['relative_errored'] = data_frame['relative_error'] > relative_error_threshold
    nb = data_frame.groupby(dispatch)['contrat_de_travail'].count()
    nb_absolute = data_frame.groupby(dispatch)['absolute_errored'].sum().astype(int)
    pct_absolute = data_frame.groupby(dispatch)['absolute_errored'].sum() /  data_frame.groupby(['categorie_salarie', 'contrat_de_travail'])['absolute_errored'].count()
    mean_absolute = data_frame.query('absolute_errored').groupby(dispatch)['absolute_error'].mean()
    max_aboslute = data_frame.groupby(dispatch)['absolute_error'].max()

    pct_relative = data_frame.groupby(dispatch)['relative_errored'].sum() /  data_frame.groupby(dispatch)['relative_errored'].count()
    mean_relative = data_frame.query('relative_errored').groupby(dispatch)['relative_error'].mean()
    max_relative = data_frame.groupby(dispatch)['relative_error'].max()


    max_aboslute
    pct_absolute

    max_relative
    pct_relative
    nb



from openfisca_france_data.erfs_fpr.get_survey_scenario import get_survey_scenario
from openfisca_france_data.reforms.inversion_directe_salaires import inversion_directe_salaires

survey_scenario = get_survey_scenario(year = 2014, reform = inversion_directe_salaires)

data_frame_by_entity = survey_scenario.create_data_frame_by_entity(
    variables = [
        'irpp',
        'maries_ou_pacses',
        'nbptr',
        'rfr',
        'revenu_assimile_salaire',
        'statut_marital',
        'weight_foyers',
        'weight_individus',
        ],
    )

famille = data_frame_by_entity['famille']
foyer_fiscal = data_frame_by_entity['foyer_fiscal']
individu = data_frame_by_entity['individu']
menage = data_frame_by_entity['menage']

# statut_occupation
statut_marital = individu.groupby('statut_marital')['weight_individus'].sum()
assert 22e6 < statut_marital[1] < 25e6
assert 30e6 < statut_marital[2] < 32e6

foyer_fiscal.groupby('maries_ou_pacses')['weight_foyers'].sum() / 1e6
foyer_fiscal.groupby('nbptr')['weight_foyers'].sum() / 1e6

assert 25e6 < (foyer_fiscal.weight_foyers).sum()

print(foyer_fiscal.groupby('nbptr').apply(lambda x: (x.irpp.astype('float') * x.weight_foyers.astype('float')).sum()))
print((foyer_fiscal.irpp.astype('float') * foyer_fiscal.weight_foyers.astype('float')).sum() / 1e9)

# survey_scenario.summarize_variable('salaire_imposable')
# salaire_imposable: 1 periods * 127126 cells * item size 4 (float32, default = 0) = 496.6K

# 2012: mean = 7295.99511719, min = 2.61648988724, max = 2135010.0, mass = 9.28e+08, default = 0.0%, median = 2.90648984909

# survey_scenario.summarize_variable('revenu_assimile_salaire')
# revenu_assimile_salaire: 1 periods * 127126 cells * item size 4 (float32, default = 0) = 496.6K

# 2012: mean = 7751.25683594, min = 2.61648988724, max = 2135010.0, mass = 9.85e+08, default = 0.0%, median = 2.90648984909



from .base import * # noqa  analysis:ignore


class noindiv(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Individu
    label = "Identifiant indvidu de l'ERFS"
    definition_period = YEAR


class idmen_original(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Menage
    label = "Identifiant ménage, lien avec l'identifiant dérivé de l'ERF"
    definition_period = YEAR


class idfoy_original(Variable):
    value_type = int
    is_period_size_independent = True
    entity = FoyerFiscal
    label = "Identifiant foyer, lien avec l'identifiant dérivé de l'ERF"
    definition_period = YEAR


class idfam_original(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Famille
    label = "Identifiant famille, lien avec l'identifiant dérivé de l'ERF"
    definition_period = YEAR



from pandas import read_csv
import numpy as np


if __name__ == '__main__':

    with open('zone_apl_2006.csv') as zone_csv:
        Z = read_csv(zone_csv, delimiter = ";")

    #% PSDC99          population sans double compte 99
    #% Pop_mun_2006    population municipale 2006

    #% M.POL99 de 1 à  4
    #% REG de    11 à 94
    #% TAU99      0 à 10
    #% TU99       0 à  8
    #% zone
    #
    grouped_5 =  Z.groupby(['TU99','TAU99','REG','POL99','Zone'], as_index=False)
    pop   = grouped_5['Pop_mun_2006'].aggregate(np.sum)

    # prepare result matrix by building empty result matrix
    res = pop.copy()
    res.pop('Zone')
    res.pop('Pop_mun_2006')

    res['zone1'] = 0
    res['zone2'] = 0
    res['zone3'] = 0
    print(res)
    print(pop.Pop_mun_2006[pop['Zone'] == 1])

    res['zone1'] = res['zone1'] + pop.Pop_mun_2006[pop['Zone']==1]
    res['zone2'] = res['zone2'] + pop.Pop_mun_2006[pop['Zone']==2]
    res['zone3'] = res['zone3'] + pop.Pop_mun_2006[pop['Zone']==3]

    print(res.to_string())

    for col in ('zone1', 'zone2', 'zone3'):
        res[col][np.isnan(res[col])] = 0

    print(res.to_string())
    res2 = res.groupby(['TU99', 'TAU99', 'REG', 'POL99'])

    final = res2.agg({
        'zone1': np.sum,
        'zone2': np.sum,
        'zone3': np.sum
        })
    final['total'] = final['zone1'] + final['zone2'] + final['zone3']
    final['proba_zone1'] = final['zone1']/final['total']
    final['proba_zone2'] = final['zone2']/final['total']
    final['proba_zone3'] = final['zone3']/final['total']

    final.pop('zone1')
    final.pop('zone2')
    final.pop('zone3')
    final.pop('total')

    final = final.reset_index()

    print(final)

    # Sanity check
    # s = final['p1'] + final['p2'] + final['p3']

    final.to_csv('./zone_apl_imputation_data.csv')



import pkg_resources
import os

import pytest

from openfisca_survey_manager.calibration import Calibration  # type: ignore


@pytest.fixture
def location() -> str:
    return pkg_resources.get_distribution("openfisca-france-data").location


def test_calibration(survey_scenario, fake_input_data, location, year: int = 2009):
    # On ititialise le survey scenario
    survey_scenario = survey_scenario(year)

    # On charge les données
    input_data = fake_input_data(year)

    # On initialise le survey scenario
    survey_scenario.init_from_data(data = dict(input_data_frame = input_data))

    # On fait la calibration
    calibration = Calibration(survey_scenario)
    calibration.parameters["method"] = "linear"
    calibration.total_population = calibration.initial_total_population * 1.123

    calibration.set_parameters("invlo", 3)
    calibration.set_parameters("up", 3)
    calibration.set_parameters("method", "logit")

    calibration.calibrate()
    pre_cal_weight = survey_scenario.calculate_variable("wprm", period = year)
    calibration.set_calibrated_weights()
    assert pre_cal_weight * 1.123 == survey_scenario.calculate_variable("wprm", period = year)




#! /usr/bin/env python


# This step requires to have R installed with StatMatch library
# You'll also need rpy2 2.3x available at
# https://bitbucket.org/lgautier/rpy2/src/511312d70346f3f66c989e35443b2823e9b56d56?at=version_2.3.x
# (the version on python website is not compatible, working correctly for the debian's testing version)


import gc
import logging
import numpy
import pandas

# problem with rpy last version is not working
try:
    import rpy2
    import rpy2.robjects.pandas2ri
except ImportError:
    rpy2 = None

from openfisca_france_data.utils import assert_variable_in_range, count_NA
from openfisca_survey_manager.temporary import temporary_store_decorator, save_hdf_r_readable
from openfisca_survey_manager.statshelpers import mark_weighted_percentiles
from openfisca_survey_manager.survey_collections import SurveyCollection


log = logging.getLogger(__name__)


def create_comparable_erf_data_frame(temporary_store = None, year = None):
    '''
    Imputation des loyers
    '''
    assert temporary_store is not None
    assert year is not None
    # Préparation des variables qui serviront à l'imputation
    # Année achèvement de l'immeuble (8 postes) aai1
    # Année achèvement de l'immeuble aai2
    menage_variables = [
        "aai1",
        "agpr",
        "cstotpr",
        "ident",
        "nat28pr",
        "nb_uci",
        "nbenfc",
        "nbpiec",
        "pol99",
        "reg",
        "so",
        "spr",
        "tau99",
        "tu99",
        "typmen5",
        "wprm",
        "zperm",
        "zracm",
        "zragm",
        "zricm",
        "zrncm",
        "ztsam",
        ]

    if year == 2008:  # Tau99 not present
        menage_variables = menage_variables.pop('tau99')

    # Travail sur la base ERF
    log.info("Préparation de la table ménage de l'ERF")
    erf_menages = temporary_store['menagem_{}'.format(year)]
    erf_menages = erf_menages[menage_variables].copy()

    erf_menages['revtot'] = (
        erf_menages.zperm +
        erf_menages.zragm +
        erf_menages.zricm +
        erf_menages.zrncm +
        erf_menages.ztsam +
        erf_menages.zracm
        )
    # Niveau de vie de la personne de référence
    erf_menages['nvpr'] = erf_menages.revtot.astype('float') / erf_menages.nb_uci.astype('float')
    erf_menages.loc[erf_menages.nvpr < 0, 'nvpr'] = 0
    erf_menages.rename(columns = dict(so = 'statut_occupation'), inplace = True)

    log.info("Preparation de la tables individus de l'ERF")
    erfindm = temporary_store['indivim_{}'.format(year)]
    erfindm = erfindm[['ident', 'dip11']][erfindm.lpr == 1].copy()

    log.info("Merging talbes menage et individus de l'ERF")
    erf = erfmenm.merge(erfindm, on = 'ident', how = 'inner')
    erf = erf.drop_duplicates('ident')

    dec, values = mark_weighted_percentiles(
        erf.nvpr.values,
        numpy.arange(1, 11),
        erf.wprm.values,
        2,
        return_quantiles = True
        )
    values.sort()
    erf['deci'] = (
        1 +
        (erf.nvpr > values[1]) +
        (erf.nvpr > values[2]) +
        (erf.nvpr > values[3]) +
        (erf.nvpr > values[4]) +
        (erf.nvpr > values[5]) +
        (erf.nvpr > values[6]) +
        (erf.nvpr > values[7]) +
        (erf.nvpr > values[8]) +
        (erf.nvpr > values[9])
        )
    assert erf.deci.isin(range(1, 11)).all()
    del dec, values
    gc.collect()

    # TODO: faire le lien avec men_vars,
    # il manque "pol99","reg","tau99" et ici on a en plus logt, 'nvpr','revtot','dip11','deci'
    erf = erf[
        [
            'ident',
            'ztsam',
            'zperm',
            'zragm',
            'zricm',
            'zrncm',
            'zracm',
            'nb_uci',
            'statut_occupation',
            'nbpiec',
            'typmen5',
            'spr',
            'nbenfc',
            'agpr',
            'cstotpr',
            'nat28pr',
            'tu99',
            'aai1',
            'wprm',
            'nvpr',
            'revtot',
            'dip11',
            'deci',
            ]
        ][erf.statut_occupation.isin(range(3, 6))].copy()

    erf.rename(
        columns = {
            'dip11': 'mdiplo',
            'nbpiec': 'hnph2',
            'nat28pr': 'mnatio',
            'nbpiec': 'hnph2',
            },
        inplace = True)

    if (erf.agpr == '.').any():
        erf = erf[erf.agpr != '.'].copy()

    erf['agpr'] = erf.agpr.astype('int')
    erf['magtr'] = 3
    erf.magtr[erf.agpr < 65] = 2
    erf.magtr[erf.agpr < 40] = 1
    assert erf.magtr.isin(range(1, 5)).all()

    assert erf.cstotpr.notnull().all()
    erf['mcs8'] = erf.cstotpr.astype('float') / 10.0
    erf.mcs8 = numpy.floor(erf.mcs8).astype('int')
    # TODO: assert erf.mcs8.isin(range(1, 9))
    assert erf.mcs8.isin(range(0, 9)).all()

    erf['mtybd'] = 0
    erf.mtybd[(erf.typmen5 == 1) & (erf.spr != 2)] = 1
    erf.mtybd[(erf.typmen5 == 1) & (erf.spr == 2)] = 2
    erf.mtybd[erf.typmen5 == 5] = 3
    erf.mtybd[erf.typmen5 == 3] = 7
    erf.mtybd[erf.nbenfc == 1] = 4
    erf.mtybd[erf.nbenfc == 2] = 5
    erf.mtybd[erf.nbenfc >= 3] = 6
    assert erf.mtybd.isin(range(0, 8)).all()
    # TODO: assert erf.mtybd.isin(range(1,8)).all(),

    # 3 logements ont 0 pièces !!
    erf.hnph2[erf.hnph2 < 1] = 1
    erf.hnph2[erf.hnph2 >= 6] = 6
    assert erf.hnph2.isin(range(1, 7)).all()

    # TODO: il reste un NA 2003
    #       il rest un NA en 2008  (Not rechecked)

    erf.mnatio[erf.mnatio == 10] = 1
    mnatio_range = range(11, 16) + range(21, 30) + range(31, 33) + range(41, 49) + range(51, 53) + [60] + [62]
    erf.mnatio[erf.mnatio.isin(mnatio_range)] = 2
    assert erf.mnatio.isin(range(1, 3)).all()

    # iaat_bis is a contraction of iaat varaible of enquête logement
    # iaat_bis DATE D'ACHEVEMENT DE LA CONSTRUCTION
    # Voir http://www.insee.fr/fr/ppp/bases-de-donnees/fichiers_detail/eec10/doc/pdf/eec10_qlogement.pdf
    erf['iaat_bis'] = 0
    erf.iaat_bis[erf.aai1.isin([1, 2, 3])] = 1
    erf.iaat_bis[erf.aai1 == 4] = 2
    erf.iaat_bis[erf.aai1 == 5] = 3
    erf.iaat_bis[erf.aai1 == 6] = 4
    erf.iaat_bis[erf.aai1 == 7] = 5
    erf.iaat_bis[erf.aai1 == 8] = 6
    # TODO: assert erf.iaat_bis.isin(range(1, 7)).all()
    assert erf.iaat_bis.isin(range(0, 7)).all()

    # Il reste un NA en 2003
    # reste un NA en 2008
    # TODO: comparer logement et erf pour être sur que cela colle ???
    erf.mdiplo[erf.mdiplo.isin([71, ""])] = 1
    erf.mdiplo[erf.mdiplo.isin([70, 60, 50])] = 2
    erf.mdiplo[erf.mdiplo.isin([41, 42, 31, 33])] = 3
    erf.mdiplo[erf.mdiplo.isin([10, 11, 30])] = 4
    # TODO: assert erf.mdiplo.isin(range(1, 5)).all()
    assert erf.mdiplo.isin(range(0, 5)).all()

    erf['tu99_recoded'] = erf.tu99.copy()
    erf.tu99_recoded[erf.tu99 == 0] = 1
    erf.tu99_recoded[erf.tu99.isin([1, 2, 3])] = 2
    erf.tu99_recoded[erf.tu99.isin([4, 5, 6])] = 3
    erf.tu99_recoded[erf.tu99 == 7] = 4
    erf.tu99_recoded[erf.tu99 == 8] = 5
    assert erf.tu99_recoded.isin(range(1, 6)).all()

    erf.loc[erf.mcs8.isin([4, 8]), 'mcs8'] = 4
    erf.loc[erf.mcs8.isin([5, 6, 7]), 'mcs8'] = 5
    # TODO: assert erf.mcs8.isin(range(1, 6)).all(), erf.mcs8.value_counts()
    assert erf.mcs8.isin(range(0, 6)).all()

    erf.wprm = erf.wprm.astype('int')

    for dropped_variable in ['agpr', 'cstotpr', 'nbenfc', 'spr', 'tu99', 'typmen5']:
        del erf[dropped_variable]
    gc.collect()

    check_variables = [
        'statut_occupation',
        'magtr',
        'mcs8',
        'mtybd',
        'hnph2',
        'mnatio',
        'iaat_bis',
        'mdiplo',
        'tu99_recoded',
        ]

    non_integer_variables_to_filter = list()
    integer_variables_to_filter = list()
    for check_variable in check_variables:
        if erf[check_variable].isnull().any():
            log.info("Des valeurs NaN sont encore présentes dans la variable {} la table ERF".format(check_variable))
            non_integer_variables_to_filter.append(check_variable)

        if (erf[check_variable] == 0).any():
            if numpy.issubdtype(erf[check_variable].dtype, numpy.integer):
                log.info(
                    "Des valeurs nulles sont encore présentes dans la variable {} la table ERF".format(check_variable))
                integer_variables_to_filter.append(check_variable)

    assert len(non_integer_variables_to_filter) == 0, non_integer_variables_to_filter
    # assert len(integer_variables_to_filter) == 0, integer_variables_to_filter

    # On vérifie au final que l'on n'a pas de doublons d'individus
    assert not erf['ident'].duplicated().any(), 'Il y a {} doublons'.format(erf['ident'].duplicated().sum())
    return erf


def create_comparable_logement_data_frame(temporary_store = None, year = None):
    assert temporary_store is not None
    assert year is not None
    logement_adresse_variables = ["gzc2"]
    logement_menage_variables = [
        "maa1at",
        "magtr",
        "mcs8",
        "mdiplo",
        "mrcho",
        "mrret",
        "mrsal",
        "mrtns",
        "mtybd",
        "muc1",
        "qex",
        "sec1",
        ]

    if year == 2003:
        logement_menage_variables.extend(["hnph2", "ident", "lmlm", "mnatior", "typse"])
        logement_adresse_variables.extend(["iaat", "ident", "tu99"])
    if year < 2010 and year > 2005:
        logement_menage_variables.extend(["idlog", "mnatio"])
        logement_adresse_variables.extend(["idlog"])  # pas de typse en 2006
        logement_logement_variables = ["hnph2", "iaat", "idlog", "lmlm", "tu99"]  # pas de typse en 2006

    # Travail sur la table logement
    # Table menage
    if year == 2003:
        year_lgt = 2003
    if year > 2005 and year < 2010:
        year_lgt = 2006

    logement_survey_collection = SurveyCollection.load(collection = 'logement')
    logement_survey = logement_survey_collection.get_survey('logement_{}'.format(year_lgt))

    log.info("Preparing logement menage table")

    try:
        logement_menage = logement_survey.get_values(
            table = "lgt_menage", variables = logement_menage_variables)
    except Exception:
        logement_menage = logement_survey.get_values(
            table = "menage1", variables = logement_menage_variables)

    logement_menage.rename(columns = {'idlog': 'ident'}, inplace = True)

    logement_menage['mrcho'].fillna(0, inplace = True)
    logement_menage['mrret'].fillna(0, inplace = True)
    logement_menage['mrsal'].fillna(0, inplace = True)
    logement_menage['mrtns'].fillna(0, inplace = True)
    logement_menage['revtot'] = (
        logement_menage['mrcho'] +
        logement_menage['mrret'] +
        logement_menage['mrsal'] +
        logement_menage['mrtns']
        )  # TODO : Virer les revenus négatifs ? mrtns :  118 revenus négatifs sur 42845 en 2006
    assert logement_menage.revtot.notnull().all()
    logement_menage['nvpr'] = 10.0 * logement_menage['revtot'] / logement_menage['muc1']

    assert logement_menage.qex.notnull().all()
    assert (logement_menage.qex > 0).all()

    dec, values = mark_weighted_percentiles(
        logement_menage['nvpr'].values,
        numpy.arange(1, 11),
        logement_menage['qex'].values,
        2,
        return_quantiles = True,
        )
    values.sort()
    logement_menage['deci'] = (
        1 +
        (logement_menage.nvpr > values[1]) +
        (logement_menage.nvpr > values[2]) +
        (logement_menage.nvpr > values[3]) +
        (logement_menage.nvpr > values[4]) +
        (logement_menage.nvpr > values[5]) +
        (logement_menage.nvpr > values[6]) +
        (logement_menage.nvpr > values[7]) +
        (logement_menage.nvpr > values[8]) +
        (logement_menage.nvpr > values[9])
        )

    del dec, values
    assert logement_menage['deci'].isin(range(1, 11)).all(), "Logement decile are out of range'"
    gc.collect()

    if year_lgt == 2006:
        log.info('Preparing logement logement table')
        try:
            lgtlgt = logement_survey.get_values(
                table = "lgt_logt", variables = logement_logement_variables)
        except Exception:
            lgtlgt = logement_survey.get_values(
                table = "logement", variables = logement_logement_variables)

        lgtlgt.rename(columns = {'idlog': 'ident'}, inplace = True)
        logement_menage = logement_menage.merge(lgtlgt, left_on = 'ident', right_on = 'ident', how = 'inner')
        del lgtlgt

    data = logement_menage[logement_menage['sec1'].isin([21, 22, 23, 24, 30])]
    del logement_menage
    gc.collect()

    if year_lgt == 2006:
        data.rename(columns = {'mnatio': 'mnatior'}, inplace = True)

    data = (data[data['mnatior'].notnull()])
    data = (data[data['sec1'].notnull()])
    data['tmp'] = data['sec1'].astype("int")
    data['tmp'][data['sec1'].isin([21, 22, 23])] = 3
    data['tmp'][data['sec1'] == 24] = 4
    data['tmp'][data['sec1'] == 30] = 5
    data['statut_occupation'] = data['tmp']
    count_NA('statut_occupation', data)
    data = (data[data['statut_occupation'].notnull()])
    logement_menage = data

    # Table adresse
    log.info("Préparation de la table adresse de l'enquête logement")

    logement_adresse = logement_survey.get_values(table = "adresse", variables = logement_adresse_variables)
    logement_adresse.rename(columns = {'idlog': 'ident'}, inplace = True)

    log.info("Fusion des tables logement et ménage de l'enquête logement")
    Logement = logement_menage.merge(logement_adresse, on = 'ident', how = 'inner')

    Logement.hnph2[Logement.hnph2 >= 6] = 6
    Logement.hnph2[Logement.hnph2 < 1] = 1
    count_NA('hnph2', Logement)
    assert Logement.hnph2.notnull().any(), "Some hnph2 are null"
#     Logement=(Logement[Logement['hnph2'].notnull()]) # Mis en comment car 0 NA pour hnph2

    # On est dans la même étape within ici et par la suite ( cf code R )
    # TODO : ici problème je transforme les 07 en 7
    # car Python considère les 0n comme des nombres octaux ( < 08 ).
    # J'espère que ce n'est pas important.
    Logement.mnatior[Logement['mnatior'].isin([0, 1])] = 1
    Logement.mnatior[Logement['mnatior'].isin([2, 3, 4, 5, 6, 7, 8, 9, 10, 11])] = 2
    count_NA('mnatior', Logement)
    assert_variable_in_range('mnatior', [1, 3], Logement)

    Logement['iaat_bis'] = 0
    Logement.iaat_bis[Logement.iaat.isin([1, 2, 3, 4, 5])] = 1  # avant 1967
    Logement.iaat_bis[Logement.iaat == 6] = 2  # 1968 - 1974
    Logement.iaat_bis[Logement.iaat == 7] = 3  # 1975 - 1981
    Logement.iaat_bis[Logement.iaat == 8] = 4  # 1982 - 1989
    Logement.iaat_bis[Logement.iaat == 9] = 5  # 1990 - 1998
    Logement.iaat_bis[Logement.iaat == 10] = 6  # après 1999
    assert Logement.iaat_bis.isin(range(1, 7)).all()

    Logement.mdiplo[Logement.mdiplo == 1] = 1
    Logement.mdiplo[Logement.mdiplo.isin([2, 3, 4])] = 2
    Logement.mdiplo[Logement.mdiplo.isin([5, 6, 7, 8])] = 3
    Logement.mdiplo[Logement.mdiplo == 9] = 4
    Logement.mdiplo[Logement.mdiplo.isnull()] = 0
    # TODO: assert Logement.mdiplo.isin(range(1, 5)).all()
    assert Logement.mdiplo.isin(range(0, 5)).all()
    Logement.mdiplo = Logement.mdiplo.astype('int')

    Logement.mtybd[Logement['mtybd'] == 110] = 1
    Logement.mtybd[Logement['mtybd'] == 120] = 2
    Logement.mtybd[Logement['mtybd'] == 200] = 3
    Logement.mtybd[Logement['mtybd'].isin([311, 321, 401])] = 4
    Logement.mtybd[Logement['mtybd'].isin([312, 322, 402])] = 5
    Logement.mtybd[Logement['mtybd'].isin([313, 323, 403])] = 6
    Logement.mtybd[Logement['mtybd'] == 400] = 7
    assert Logement.mtybd.isin(range(1, 8)).all()
    Logement.mtybd = Logement.mtybd.astype('int')

    Logement['tu99_recoded'] = Logement['tu99'].copy()
    count_NA('tu99', Logement)
    Logement.tu99_recoded[Logement['tu99'] == 0] = 1
    Logement.tu99_recoded[Logement['tu99'].isin([1, 2, 3])] = 2
    Logement.tu99_recoded[Logement['tu99'].isin([4, 5, 6])] = 3
    Logement.tu99_recoded[Logement['tu99'] == 7] = 4
    Logement.tu99_recoded[Logement['tu99'] == 8] = 5
    count_NA('tu99_recoded', Logement)
    assert_variable_in_range('tu99_recoded', [1, 6], Logement)

    Logement.gzc2[Logement['gzc2'] == 1] = 1
    Logement.gzc2[Logement['gzc2'].isin([2, 3, 4, 5, 6])] = 2
    Logement.gzc2[Logement['gzc2'] == 7] = 3
    count_NA('gzc2', Logement)
    # TODO: assert_variable_in_range('gzc2', [1, 4], Logement)

    Logement.magtr[Logement['magtr'].isin([1, 2])] = 1
    Logement.magtr[Logement['magtr'].isin([3, 4])] = 2
    Logement.magtr[Logement['magtr'] == 5] = 3
    assert Logement.magtr.isin(range(1, 4)).all()

    Logement['mcs8'][Logement['mcs8'] == 1] = 1
    Logement['mcs8'][Logement['mcs8'] == 2] = 2
    Logement['mcs8'][Logement['mcs8'] == 3] = 3
    Logement['mcs8'][Logement['mcs8'].isin([4, 8])] = 4
    Logement['mcs8'][Logement['mcs8'].isin([5, 6, 7])] = 5
    assert Logement.mcs8.isin(range(1, 6)).all()

    Logement['logloy'] = numpy.log(Logement['lmlm'].values)
    kept_variables = [
        'deci',
        'hnph2',
        'iaat_bis',
        # 'ident',
        'lmlm',
        'statut_occupation',
        'magtr',
        'mcs8',
        'mdiplo',
        'mtybd',
        'qex',
        'tu99_recoded',
        ]

    logement = Logement[kept_variables].copy()
    # logement.rename(columns = {'qex': 'wprm'}, inplace = True)
    return logement


@temporary_store_decorator(file_name = 'erfs')
def imputation_loyer(temporary_store = None, year = None):
    assert temporary_store is not None
    assert year is not None

    erf = create_comparable_erf_data_frame(temporary_store = temporary_store, year = year)

    logement = create_comparable_logement_data_frame(temporary_store = temporary_store, year = year)
    log.info("dropping {} observations form logement".format(logement.lmlm.isnull().sum()))
    logement = logement.loc[logement.lmlm.notnull()].copy()

    allvars = ['statut_occupation', 'hnph2', 'iaat_bis', 'mdiplo', 'mtybd', 'tu99_recoded', 'magtr', 'mcs8', 'deci']

    # TODO keep the variable indices
    erf = erf[allvars + ['wprm', 'ident']].copy()

    for variable in allvars:
        erf_unique_values = set(erf[variable].unique())
        logement_unique_values = set(logement[variable].unique())
        if not erf_unique_values <= logement_unique_values:
            print('''
                {} span wrong
                erf: {},
                logement: {}
                concerns {} observations
                '''.format(
                    variable,
                    erf_unique_values,
                    logement_unique_values,
                    erf[variable].isin(erf_unique_values - logement_unique_values).sum()
                    )
                )

    # print("dropping {} erf observations".format(len(erf.query('iaat_bis == 0 | mtybd == 0 | mcs8 == 0'))))
    erf = erf.query('iaat_bis != 0 & mtybd != 0 & mcs8 != 0').copy()

    for variable in allvars:
        erf_unique_values = set(erf[variable].unique())
        logement_unique_values = set(logement[variable].unique())
        assert erf_unique_values <= logement_unique_values

    classes = ['tu99_recoded', 'deci']
    matchvars = list(set(allvars) - set(classes))

    # Clean for unicode column names
    erf.rename(
        columns = dict([(col, str(col)) for col in erf.columns]),
        inplace = True
        )

    save_hdf_r_readable(erf.astype('int32'), file_name = "erf_imputation")
    save_hdf_r_readable(logement.astype('int32'), file_name = "logement_imputation")

    import subprocess
    import os
    os.path.abspath
    imputation_loyer_R = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'imputation_loyer.R')
    temporary_store_directory = os.path.dirname(temporary_store.filename)
    print(imputation_loyer_R)
    try:
        subprocess.check_call(['/usr/bin/Rscript', imputation_loyer_R, temporary_store_directory])
    except WindowsError:
        subprocess.check_call(['C:/Program Files/R/R-2.15.1/bin/x64/Rscript.exe',
            imputation_loyer_R, temporary_store_directory], shell = True)

    fill_erf_nnd = pandas.read_hdf(os.path.join(temporary_store_directory, 'imputation.h5'), 'fill_erf_nnd')
    assert set(fill_erf_nnd.ident) == set(erf.ident.astype('int32'))

#    rpy2.robjects.pandas2ri.activate()  # Permet à rpy2 de convertir les dataframes
#    sm = rpy2.robjects.packages.importr("StatMatch")  # Launch R you need to have StatMatch installed in R
#
#    out_nnd = sm.NND_hotdeck(
#        data_rec = erf.astype('int32'),
#        data_don = logement.astype('int32'),
#        match_vars = rpy2.robjects.vectors.StrVector(matchvars),
#        don_class = rpy2.robjects.vectors.StrVector(classes),
#        dist_fun = "Gower",
#        )
#
#
#    mtc_ids = out_nnd[0].reshape(len(erf), 2)  # Essential to avoid a bug in next line
#
#    fill_erf_nnd = sm.create_fused(
#        data_rec = erf.astype('int32'),
#        data_don = logement.astype('int32'),
#        mtc_ids = mtc_ids,
#        z_vars = "lmlm",
#        )

    fill_erf_nnd.rename(columns = {'lmlm': 'loyer'}, inplace = True)
    loyers_imputes = fill_erf_nnd[['ident', 'loyer']].copy()
    menagem = temporary_store['menagem_{}'.format(year)]
    for loyer_var in ['loyer_x', 'loyer_y', 'loyer']:
        if loyer_var in menagem.columns:
            del menagem[loyer_var]

    menagem = menagem.merge(loyers_imputes, on = 'ident', how = 'left')
    assert 'loyer' in menagem.columns, "La variable loyer n'est pas présente dans menagem"

    temporary_store['menagem_{}'.format(year)] = menagem
    return


if __name__ == '__main__':
    import sys
    logging.basicConfig(level = logging.INFO, stream = sys.stdout)
    year = 2009

    imputation_loyer(year = year)

    # menagem, erf, fill_erf_nnd, logement = imputation_loyer(year = year)



import pytest


from openfisca_france_data import france_data_tax_benefit_system  # type: ignore
from openfisca_france_data.aggregates import FranceAggregates as Aggregates  # type: ignore
from openfisca_france_data.erfs_fpr.scenario import (  # type: ignore
    ErfsFprSurveyScenario,
    )


@pytest.mark.skip(
    reason = "FileNotFoundError: [Errno 2] No such file or directory: '/opt/hostedtoolcache/Python/3.9.9/x64/lib/python3.9/site-packages/openfisca_france_data/assets/agregats_tests_taxipp_2_0.xlsx'",
    )
def test_erfs_survey_simulation(survey_scenario, fake_input_data, year: int = 2009):
    # On ititialise le survey scenario
    survey_scenario = ErfsFprSurveyScenario.create(year)

    # On charge les données
    input_data = fake_input_data(year)

    # On initialise le survey scenario
    survey_scenario.init_from_data(data = dict(input_data_frame = input_data))

    # On calcule les agrégats
    aggregates = Aggregates(survey_scenario = survey_scenario, target_source = 'taxipp')
    aggregates.compute_aggregates(use_baseline = False)
    return aggregates.base_data_frame


@pytest.mark.skip(
    reason = "FileNotFoundError: [Errno 2] No such file or directory: '/opt/hostedtoolcache/Python/3.9.9/x64/lib/python3.9/site-packages/openfisca_france_data/assets/agregats_tests_taxipp_2_0.xlsx'",
    )
def test_erfs_fpr_aggregates_reform(fake_input_data, year:int = 2013):
    survey_scenario = ErfsFprSurveyScenario.create(
        year = year,
        reform_key = 'plf2015',
        baseline_tax_benefit_system = france_data_tax_benefit_system,
        )
    # On charge les données
    input_data = fake_input_data(year)

    # On initialise le survey scenario
    survey_scenario.init_from_data(data = dict(input_data_frame = input_data))

    aggregates = Aggregates(survey_scenario = survey_scenario, target_source = 'taxipp')
    base_data_frame = aggregates.compute_aggregates()
    return aggregates, base_data_frame
